'''
Copyright (c) 2020 epiphani, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
'''
import sys
import appsync_subscription_manager as asm
import warrant
import os
from datetime import datetime
import threading
import pprint
import ecube.environments as environments
import time
import requests
import hashlib
import json
import traceback
import signal
import ecube.gql_operations.Mutations as Mutations
import ecube.gql_operations.Queries as Queries

ENVIRONMENTS = environments.ENVIRONMENTS
ARTIBOT_USERNAME = "artibot"

# Read GQL URL
LOCAL_GQL_HOST = os.environ.get('LOCAL_GQL_HOST', "epic-sandbox")
LOCAL_GQL_PORT = os.environ.get('LOCAL_GQL_PORT', "4000")
LOCAL_GQL_FRAG = "%s:%s" % (LOCAL_GQL_HOST, LOCAL_GQL_PORT)
GQL_PSK = os.environ.get("GQL_PSK", None)

LOG_DIR = "/var/log/epiphani/"

class WorkerPool():
    def apply_async(self, t_func, t_args):
        threading.Thread(target=t_func, args=t_args).start()

class GetObjError(Exception):
     def __init__(self, *args, **kwargs):
         default_message = 'This is a default message!'

         # if no arguments are passed set the first positional argument
         # to be the default message. To do that, we have to replace the
         # 'args' tuple with another one, that will only contain the message.
         # (we cannot do an assignment since tuples are immutable)
         if not (args or kwargs): args = (default_message,)

         # Call super constructor
         super(GetObjError, self).__init__(*args, **kwargs)

class ListObjectsError(Exception):
     def __init__(self, *args, **kwargs):
         default_message = 'This is a default message!'

         # if no arguments are passed set the first positional argument
         # to be the default message. To do that, we have to replace the
         # 'args' tuple with another one, that will only contain the message.
         # (we cannot do an assignment since tuples are immutable)
         if not (args or kwargs): args = (default_message,)

         # Call super constructor
         super(ListObjectsError, self).__init__(*args, **kwargs)

class DeleteObjectError(Exception):
     def __init__(self, *args, **kwargs):
         default_message = 'This is a default message!'

         # if no arguments are passed set the first positional argument
         # to be the default message. To do that, we have to replace the
         # 'args' tuple with another one, that will only contain the message.
         # (we cannot do an assignment since tuples are immutable)
         if not (args or kwargs): args = (default_message,)

         # Call super constructor
         super(DeleteObjectError, self).__init__(*args, **kwargs)

class InsertObjectError(Exception):
     def __init__(self, *args, **kwargs):
         default_message = 'This is a default message!'

         # if no arguments are passed set the first positional argument
         # to be the default message. To do that, we have to replace the
         # 'args' tuple with another one, that will only contain the message.
         # (we cannot do an assignment since tuples are immutable)
         if not (args or kwargs): args = (default_message,)

         # Call super constructor
         super(InsertObjectError, self).__init__(*args, **kwargs)

class UpdateObjectError(Exception):
     def __init__(self, *args, **kwargs):
         default_message = 'This is a default message!'

         # if no arguments are passed set the first positional argument
         # to be the default message. To do that, we have to replace the
         # 'args' tuple with another one, that will only contain the message.
         # (we cannot do an assignment since tuples are immutable)
         if not (args or kwargs): args = (default_message,)

         # Call super constructor
         super(UpdateObjectError, self).__init__(*args, **kwargs)

class Logger():
    DEBUG = 1
    INFO = 2
    ERROR = 3
    OFF = 4

    def __init__(self, log_to_file=True, log_file=None):
        self.my_pid = os.getpid()
        self.log_to_file = log_to_file
        self.log_file = log_file
        if log_file == None:
            log_file = LOG_DIR + "ExecRunbook.%d.log" % (self.my_pid)
        #print("Initializing logger for PID: %d" % (self.my_pid))

        if self.log_to_file:
            if log_file == None:
                os.mkdir(LOG_DIR)
            self.log_file = open(log_file, "w")
            self.log_level = Logger.OFF
            self.write = self.log_file.write
            self.flush = self.log_file.flush
            self.stream = self.log_file
        else:
            self.write = sys.stdout.write
            self.flush = sys.stdout.flush
            self.stream = sys.stdout

    def is_valid_log_level(self, log_level):
        l = False
        try:
            if not ((log_level < Logger.DEBUG) or (log_level > Logger.OFF)):
                l = True
        except:
            pass

        return l

    def set_log_level(self, log_level):
        if not self.is_valid_log_level(log_level):
            print("ERROR: Trying to set Invalid log level: %r" % (log_level))
            sys.exit(-1)
        self.log_level = log_level

    def get_log_level(self):
        return self.log_level

    def get_log_level_str(self, log_level):
        if log_level == Logger.OFF:
            return "OFF"
        elif log_level == Logger.DEBUG:
            return "DEBUG"
        elif log_level == Logger.INFO:
            return "INFO"
        elif log_level == Logger.ERROR:
            return "ERROR"
        else:
            return "INVALID_LOG_LEVEL(%s)" % (str(log_level))

    def log(self, log_level, log_str):
        if self.is_valid_log_level(log_level):
            if log_level >= self.log_level:
                self.write("%s: %d: %s: %r\n" % (datetime.now().strftime("%m/%d/%Y, %H:%M:%S"), self.my_pid, self.get_log_level_str(log_level), log_str))
        else:
            self.write("%s: %d: %s: %r\n" % (datetime.now().strftime("%m/%d/%Y, %H:%M:%S"), self.my_pid, self.get_log_level_str(log_level), log_str))
        self.flush()

    def pprint(self, log_level, log_obj):
        if self.is_valid_log_level(log_level):
            if log_level >= self.log_level:
                pprint.pprint(log_obj, stream=self.stream)
                self.write("\n")
        else:
            self.write("%d: %r\n" % (self.my_pid, self.get_log_level_str(log_level)))
            pprint.pprint(log_obj, stream=self.stream)
            self.write("\n")
        self.flush()

# calculate and return the md5sum of the contents of a given file
def get_file_md5sum(file_name):
    with open(file_name, "rb") as f:
        file_hash = hashlib.md5()
        t = f.read(8192)
        while t:
            file_hash.update(t)
            t = f.read(8192)

    return file_hash.hexdigest()

def initEnv(current_env, logger):
    users = get_env_var(current_env, 'USERS')
    gql_endpoint = get_env_var(current_env, "GRAPHQL_API_ENDPOINT")
    user_list = []
    user_dict = {}
    username_dict = {}

    # Initialize the environment
    init_environment(current_env, users, user_list, user_dict, username_dict, logger)
    myauth = users[0]['auth']

    ret = {
        'id_token': myauth.id_token,
        'username': myauth.username,
        'current_env': current_env,
        'endpoint': gql_endpoint
    }
    return ret

def setup_env_and_subscribe(sub_list=None, appsync_sub_mgrs_map=None, logger=None,
    on_connection_error=None, on_error=None, on_sub_error=None,
    on_close=None, current_env=None, on_subscription_success=None,
    use_local_instance=False):
    tmp_dict = {}
    user_list = []
    user_dict = {}
    username_dict = {}

    # Setup vars for the environment
    users = get_env_var(current_env, 'USERS')
    gql_endpoint = get_env_var(current_env, "GRAPHQL_API_ENDPOINT")

    # Initialize the environment
    init_environment(current_env, users, user_list, user_dict, username_dict, logger)

    for tmp_user in users:
        tmp_dict[tmp_user['username']] = tmp_user['auth'].id_token

    username = users[0]['username']
    id_token = users[0]['auth'].id_token

    check_and_subscribe(endpoint=gql_endpoint, endpoint_user_dict=tmp_dict,
        username=username, id_token=id_token, use_local_instance=use_local_instance,
        sub_list=sub_list, appsync_sub_mgrs_map=appsync_sub_mgrs_map, logger=logger,
        on_connection_error=on_connection_error, on_error=on_error, on_sub_error=on_sub_error,
        on_close=on_close, current_env=current_env, on_subscription_success=on_subscription_success)

def cf_on_close(cb_data):
    # Remove the current subscription manager and create
    # a new one
    ep = cb_data['endpoint']
    logger = cb_data['kwargs']['logger']
    kwargs = cb_data['kwargs']
    logger.log(Logger.ERROR, "CF: Got Socket close for EP: %s" % (ep))

    logger.log(Logger.INFO, "CF: Calling APP Socket 'on-close'")
    kwargs['on_close'](cb_data)
    logger.log(Logger.INFO, "CF: Cleaning up EP: %s" % (ep))
    kwargs['appsync_sub_mgrs_map'].pop(ep, None)

    logger.log(Logger.INFO, "CF: Re-Subscribing EP: %s" % (ep))
    setup_env_and_subscribe(sub_list=kwargs['sub_list'],
        appsync_sub_mgrs_map=kwargs['appsync_sub_mgrs_map'], logger=kwargs['logger'],
        on_connection_error=kwargs['on_connection_error'],
        on_error=kwargs['on_error'], on_sub_error=kwargs['on_sub_error'],
        on_close=kwargs['on_close'], current_env=kwargs['current_env'],
        on_subscription_success=kwargs['on_subscription_success'],
        use_local_instance=kwargs['use_local_instance'])

'''
Since we store the args that are passed to check & subscribe, we
want to make sure that all the arguments are KW args so that they
can be stored & later passed back to this function on a socket
close for reconnection.
ALL THE ARGS SHOULD BE KWARGS.
'''
def check_and_subscribe(endpoint=None, endpoint_user_dict=None, username=None,
    id_token=None, sub_list=None, appsync_sub_mgrs_map=None, logger=None,
    on_connection_error=None, on_error=None, on_sub_error=None, on_close=None,
    current_env=None, on_subscription_success=None, use_local_instance=False):
    # MUST BE THE VERY FIRST LINE IN THIS FUNCTION!
    cur_args = locals()

    if endpoint in appsync_sub_mgrs_map:
        logger.log(Logger.DEBUG, "Sub Mgr already exists for endpoint: %s user: %s" % (endpoint, username))
        return False

    tmp_cb_data = {
        'current_env': current_env,
        'endpoint': endpoint,
        'username': username,
        'id_token': id_token,
        'endpoint_user_dict': endpoint_user_dict,
        'kwargs': cur_args,
        'use_local_instance': use_local_instance,
        'gql_psk': GQL_PSK
    }

    my_mgr = asm.AppSyncSubscriptionManager(id_token = id_token,
        appsync_api_id = endpoint.split('/')[2].split('.')[0],
        on_connection_error = on_connection_error,
        on_error = on_error,
        on_close = cf_on_close,
        use_local_instance = use_local_instance,
        cb_data = tmp_cb_data)

    sub_dict = {}

    for tmp_set in sub_list:
        sub_query = tmp_set[0]
        on_message = tmp_set[1]
        if len(tmp_set) == 3:
            sub_filter = tmp_set[2]
        else:
            sub_filter = {}

        my_sub = my_mgr.subscribe(sub_query, on_message, on_sub_error,
            on_subscription_success, sub_filter=sub_filter)
        sub_dict[my_sub.get_id()] = my_sub

    tmp_cb_data['subscriptions'] = sub_dict
    tmp_cb_data['manager'] = my_mgr

    appsync_sub_mgrs_map[endpoint] = tmp_cb_data

    x = threading.Thread(target=my_mgr.run_forever, args=())
    x.start()
    return True

def gql_main_loop(current_env, logger, users, blacklisted_tokens,
    user_list, user_dict, username_dict, sleep_time, sub_list,
    appsync_sub_mgrs_map, on_error,
    on_sub_error, on_connection_error, on_close,
    on_subscription_success, use_local_instance=False):
    global ENVIRONMENTS
    init_done = False
    env_list = []

    set_env_var(current_env, 'USERS', users)

    try:
      while True:
        if (use_local_instance or current_env) and not init_done:
            if use_local_instance or (current_env in ENVIRONMENTS):
                setup_env_and_subscribe(sub_list=sub_list, appsync_sub_mgrs_map=appsync_sub_mgrs_map, logger=logger,
                    on_connection_error=on_connection_error, on_error=on_error, on_sub_error=on_sub_error,
                    on_close=on_close, current_env=current_env, on_subscription_success=on_subscription_success,
                    use_local_instance=use_local_instance)
            else:
                logger.log(Logger.ERROR, "Could not find CURRENT_ENV: %s" % (current_env))
            init_done = True
        elif not current_env and not init_done:
            # Remove PROD to be processed in dev
            if 'PROD' in env_list:
                logger.log(Logger.INFO, "Removing PROD from env list...")
                env_list.remove('PROD')

            for env in env_list:
                if env in ENVIRONMENTS:
                    setup_env_and_subscribe(sub_list=sub_list, appsync_sub_mgrs_map=appsync_sub_mgrs_map, logger=logger,
                        on_connection_error=on_connection_error, on_error=on_error, on_sub_error=on_sub_error,
                        on_close=on_close, current_env=env, on_subscription_success=on_subscription_success)
                else:
                    logger.log(Logger.ERROR, "Could not find ENV: %s" % (env))

            init_done = True

        # logger.log(Logger.DEBUG, "Sleeping for %d secs..." % sleep_time)
        time.sleep(sleep_time)
    except KeyboardInterrupt as e:
        os.kill(os.getpid(), signal.SIGTERM)

def get_env_var(env, var):
    return ENVIRONMENTS[env][var]

def set_env_var(env, var, val):
    ENVIRONMENTS[env][var] = val

def init_environment(env, users, user_list, user_dict, username_dict, logger):
    # First clean the user_list (in case its a re-init)
    del user_list[:]

    # Authenticate all the users...
    for user in users:
        user_list.append(user['username'])
        # create a cognito user object
        u = warrant.Cognito(get_env_var(env, 'COGNITO_USER_POOL_ID'),
            get_env_var(env, 'COGNITO_USER_POOL_CLIENT_ID'),
            username=user['username'])

        try:
            # Try to authenticate the user
            u.authenticate(password=user['passwd'])
        except Exception as e:
            # Authentication failed...
            logger.log(Logger.ERROR, "%r" % (e))
            sys.exit(2)

        # Save the authenticated cognito user object
        user['auth'] = u

        # get the user ID by decoding the id_token
        decoded = u.verify_token(u.id_token, 'id_token', 'id')
        user['id'] = decoded['sub']
        user['decoded_id_token'] = decoded

        # Save the warrant cognito user object in the USER ID MAP
        user_dict[decoded['sub']] = u
        username_dict[user['username']] = u

    set_env_var(env, 'USERNAME_DICT', username_dict)
    set_env_var(env, 'USER_DICT', user_dict)

def check_user_token(current_env, endpoint, username, username_dict):
    env_endpoint = get_env_var(current_env, "GRAPHQL_API_ENDPOINT")

    if env_endpoint == endpoint:
        a = username_dict.get(username, None)

        if a:
            return a.id_token

    return None

'''
This function takes a function pointer and executes it.
If the execution results in a 'token expired' error, it
will try to re-authenticate the 'username' with its
password from the current_env & if successful, will use
that new token and retry the function call.

ARGUMENTS:

func: Function that should be called and retried
args: Arguments for 'func' or () if no arguments
kwargs: Keyword arguments for 'func' or {} if no keyword arguments
current_env: Current environment string
username: Username that should be used for retrying if 'func'
          fails due to 'token expired' error
id_token_index: Index of the ID token in 'args' or -1
endpoint_index: Index of the endpoint URL in 'args' above or -1

The next 3 arguments should be passed from
when the environment was originally initialized
user_list: list of users in this environment
user_dict: dictionary of users keyed of user id
username_dict: dictionary of users keyed of username

logger: Instance of 'Logger' class to be used for logs
'''
def execute_function_with_retry(func, args, kwargs, current_env,
    username, id_token_index, endpoint_index,
    user_list, user_dict, username_dict, logger,
    use_local_instance=False):
    retried_already = False

    if use_local_instance:
        kwargs.update({'use_local_instance': use_local_instance})
        current_env = None

    while True:
        try:
            # Try to get an updated token before calling the function
            # But not if this is a local instance
            if (not use_local_instance) and current_env and endpoint_index >= 0:
                new_token = check_user_token(current_env, args[endpoint_index], username, username_dict)

                if new_token:
                    args = args[:id_token_index] + (new_token,) + args[(id_token_index + 1):]

            return func(*args, **kwargs)
        except Exception as e:
            err_str = str(e)

            if use_local_instance:
                logger.log(Logger.ERROR, "Using local instance, cannot retry")
                raise

            if not current_env:
                logger.log(Logger.ERROR, "No ENV set, cannot retry")
                raise

            # Trying to handle more cases of unauthorized access. Most of the times
            # we get reason as "token is expired" but sometimes we also get "unauthorized".
            if "code: 401 reason: unauthorized" in err_str.lower():
                if not retried_already:
                    retried_already = True
                    logger.log(Logger.ERROR, "%s failed, %s, retrying" % (func.__name__, str(e)))
                    try:
                        init_environment(current_env, get_env_var(current_env, 'USERS'),
                            user_list, user_dict, username_dict, logger)
                    except Exception as e:
                        logger.log(Logger.ERROR, "Retry failed: env initialization failed")
                        raise

                    # rebuild args with new id_token
                    if username in username_dict:
                        logger.log(Logger.DEBUG, "Updated token for user: " + username)
                        args = args[:id_token_index] + (username_dict[username].id_token,) + args[(id_token_index + 1):]
                    else:
                        logger.log(Logger.ERROR, "Could not find USER: %s" % (username))
                        raise
                else:
                    logger.log(Logger.ERROR, "Already tried once, giving up")
                    raise
            else:
                logger.log(Logger.ERROR, "Unhandled error, cannot retry")
                raise

# Get the Auth header for passing to a requests library call (GET, POST, etc.)
def get_auth_header(id_token, use_local_instance=False):
    if not use_local_instance:
        return {"Authorization": id_token}
    elif GQL_PSK:
        return {"X-Api-Key": GQL_PSK}
    else:
        raise ValueError("No Pre Shared Key set for local instance authentication")

# Remove an object of a given model
def remove_obj(endpoint, id_token, model_name, obj,
    custom_query=None, mutations_module=None,
    use_local_instance=False):
    # Get the header
    auth_header = get_auth_header(id_token, use_local_instance=use_local_instance)

    if not mutations_module:
        mutations_module = Mutations

    # Set the input for delete object query
    vars = {"input": {"id": obj['id']}}
    updated_model_name = model_name[0].upper() + model_name[1:]
    connector_name = "delete" + updated_model_name

    if custom_query:
        connector = custom_query
    else:
        connector = mutations_module.MUTATIONS[connector_name.lower()]

    # Send the POST request.  All queries & mutations are sent
    # as a post request in aws-amplify/graphql, in JSON format.
    try:
        r = requests.post(endpoint, headers=auth_header,
            json={"query": connector, "variables": vars})
        res = json.loads(r.content)
    except Exception as e:
        s = "Delete object request failed: " + str(e)
        raise DeleteObjectError(s)

    if r.status_code == 200:
        # Even if the return code was 200, we could still have errors.
        if 'errors' in res:
            s = "Failed to delete object: " + updated_model_name \
                + " ERROR: " + res['errors'][0]['message']
            raise DeleteObjectError(s)
    else:
        s = "ERROR: Failed to delete object: %s CODE: %d REASON: %s MSG: %s" % (updated_model_name, r.status_code, r.reason, res['errors'][0]['message'])
        raise DeleteObjectError(s)

# Insert an object of a given model
def insert_obj(endpoint, id_token, model_name, obj,
    custom_query=None, mutations_module=None,
    use_local_instance=False):
    # Get the header
    auth_header = get_auth_header(id_token, use_local_instance=use_local_instance)

    if not mutations_module:
        mutations_module = Mutations

    # Set the input for insert object query
    vars = {"input": obj}
    updated_model_name = model_name[0].upper() + model_name[1:]
    connector_name = "create" + updated_model_name

    if custom_query:
        connector = custom_query
    else:
        connector = mutations_module.MUTATIONS[connector_name.lower()]

    # Send the POST request.  All queries & mutations are sent
    # as a post request in aws-amplify/graphql, in JSON format.
    try:
        r = requests.post(endpoint, headers=auth_header,
            json={"query": connector, "variables": vars})
        res = json.loads(r.content)
    except Exception as e:
        s = "Insert object request failed: " + str(e)
        raise InsertObjectError(s)

    if r.status_code == 200:
        # Even if the return code was 200, we could still have errors.
        if 'errors' in res:
            s = "Failed to insert object: " + updated_model_name \
                + " ERROR: " + res['errors'][0]['message']
            raise InsertObjectError(s)
        else:
            return res['data'][connector_name]
    else:
        s = "ERROR: Failed to insert object: %s CODE: %d REASON: %s MSG: %s" % (updated_model_name, r.status_code, r.reason, res['errors'][0]['message'])
        raise InsertObjectError(s)

# Update an object of a given model
def update_obj(endpoint, id_token, model_name, obj,
    custom_query=None, mutations_module=None,
    use_local_instance=False):
    # Get the header
    auth_header = get_auth_header(id_token, use_local_instance=use_local_instance)

    # Set the input for update object query
    vars = {"input": obj}
    updated_model_name = model_name[0].upper() + model_name[1:]
    connector_name = "update" + updated_model_name

    if not mutations_module:
        mutations_module = Mutations

    if custom_query:
        connector = custom_query
    else:
        connector = mutations_module.MUTATIONS[connector_name.lower()]

    # Send the POST request.  All queries & mutations are sent
    # as a post request in aws-amplify/graphql, in JSON format.
    try:
        r = requests.post(endpoint, headers=auth_header,
            json={"query": connector, "variables": vars})
        res = json.loads(r.content)
    except Exception as e:
        s = "Update object request failed: " + str(e)
        raise UpdateObjectError(s)

    if r.status_code == 200:
        # Even if the return code was 200, we could still have errors.
        if 'errors' in res:
            s = "Failed to update object: " + updated_model_name \
                + " ERROR: " + res['errors'][0]['message']
            raise UpdateObjectError(s)
        else:
            return res['data'][connector_name]
    else:
        s = "ERROR: Failed to update object: %s CODE: %d REASON: %s MSG: %s" % (updated_model_name, r.status_code, r.reason, res['errors'][0]['message'])
        raise UpdateObjectError(s)

def update_obj_custom(endpoint, id_token, model_name, obj,
    connector, connector_name, use_local_instance=False):
    # Get the header
    auth_header = get_auth_header(id_token, use_local_instance=use_local_instance)

    # Set the input for update object query
    vars = {"input": obj}
    updated_model_name = model_name[0].upper() + model_name[1:]
   
    # Send the POST request.  All queries & mutations are sent
    # as a post request in aws-amplify/graphql, in JSON format.
    try:
        r = requests.post(endpoint, headers=auth_header,
            json={"query": connector, "variables": vars})
        res = json.loads(r.content)
    except Exception as e:
        s = "Update object request failed: " + str(e)
        raise UpdateObjectError(s)

    if r.status_code == 200:
        # Even if the return code was 200, we could still have errors.
        if 'errors' in res:
            s = "Failed to update object: " + updated_model_name \
                + " ERROR: " + res['errors'][0]['message']
            raise UpdateObjectError(s)
        else:
            return res['data'][connector_name]
    else:
        s = "ERROR: Failed to update object: %s CODE: %d REASON: %s MSG: %s" % (updated_model_name, r.status_code, r.reason, res['errors'][0]['message'])
        raise UpdateObjectError(s)

# Get a specific object of a given model with specified field as key
def get_obj(endpoint, id_token, model_name, vars,
            secondaryKeyFunction=False, custom_query=None,
            queries_module=None, use_local_instance=False):
    # Get the header
    auth_header = get_auth_header(id_token, use_local_instance=use_local_instance)

    if not queries_module:
        queries_module = Queries

    if secondaryKeyFunction:
        connector_name = model_name
        updated_model_name = model_name
    else:
        updated_model_name = model_name[0].upper() + model_name[1:]
        connector_name = "get" + updated_model_name

    if custom_query:
        connector = custom_query
    else:
        connector = queries_module.QUERIES[connector_name.lower()]

    # Send the POST request.  All queries & mutations are sent
    # as a post request in aws-amplify/graphql, in JSON format.
    try:
        r = requests.post(endpoint, headers=auth_header,
                json={"query": connector, "variables": vars})

        res = json.loads(r.content)
    except Exception as e:
        s = "Get obj request failed: " + str(e)
        raise GetObjError(s)

    if r.status_code == 200:
        # Even if the return code was 200, we could still have errors.
        if not 'errors' in res:
            if not secondaryKeyFunction:
                return res['data'][connector_name]

            if len(res['data'][connector_name]['items']) == 1:
                return res['data'][connector_name]['items'][0]
            else:
                s = "Could not find object: secondaryKeyFunction: %s" % (connector_name)
                raise GetObjError(s)
        else:
            s = "Failed to get objects: " + updated_model_name \
                + " ERROR: " + res['errors'][0]['message']
            raise GetObjError(s)
    else:
        s = "ERROR: Failed to get objects: %s CODE: %d REASON: %s MSG: %s" % (updated_model_name, r.status_code, r.reason, res['errors'][0]['message'])
        raise GetObjError(s)

# Get all the objects of a given model from a given starting point
def get_objs(endpoint, starting_from, id_token, model_name, filter,
             secondaryKeyFunction=False, custom_query=None,
             queries_module=None, use_local_instance=False):
    # Get the header
    auth_header = get_auth_header(id_token, use_local_instance=use_local_instance)

    if not queries_module:
        queries_module = Queries

    # Set the input for list objects query
    vars = {"limit": 100, "nextToken": starting_from}

    if filter:
        vars['filter'] = filter

    if secondaryKeyFunction:
        connector_name = model_name
        updated_model_name = model_name
    else:
        updated_model_name = model_name[0].upper() + model_name[1:]
        connector_name = "list" + updated_model_name + "s"

    if custom_query:
        connector = custom_query
    else:
        connector = queries_module.QUERIES[connector_name.lower()]

    # Send the POST request.  All queries & mutations are sent
    # as a post request in aws-amplify/graphql, in JSON format.
    try:
        r = requests.post(endpoint, headers=auth_header,
            json={"query": connector, "variables": vars})
        res = json.loads(r.content)
    except Exception as e:
        s = "List objects request failed: " + str(e)
        raise ListObjectsError(s)

    if r.status_code == 200:
        # Even if the return code was 200, we could still have errors.
        if not 'errors' in res:
            #print "Got Connectors"
            #print res['data']['listConnectorss']['items']
            return (res['data'][connector_name]['items'],
                    res['data'][connector_name]['nextToken'])
        else:
            s = "Failed to get objects: " + updated_model_name \
                + " ERROR: " + res['errors'][0]['message']
            raise ListObjectsError(s)
    else:
        s = "ERROR: Failed to get objects: %s CODE: %d REASON: %s MSG: %s" % (updated_model_name, r.status_code, r.reason, res['errors'][0]['message'])
        raise ListObjectsError(s)

def get_model_objects(endpoint, id_token, model_name, filter,
        secondaryKeyFunction=False, custom_query=None,
        queries_module=None, use_local_instance=False):
    next_token = None
    obj_list = []

    while True:
        try:
            tmp_obj_list, next_token = get_objs(endpoint, next_token,
                id_token, model_name, filter,
                secondaryKeyFunction=secondaryKeyFunction,
                custom_query=custom_query,
                queries_module=queries_module,
                use_local_instance=use_local_instance)
            obj_list = obj_list + tmp_obj_list

            if not next_token:
                return obj_list
        except ListObjectsError as e:
            print("ERROR: %s" % (str(e)))
            raise

    return obj_list

def remove_model_objects(endpoint, id_token, model_name, obj_list,
    custom_query=None, mutations_module=None):
    for tmp_obj in obj_list:
        try:
            remove_obj(endpoint, id_token, model_name, tmp_obj,
                custom_query=custom_query, mutations_module=mutations_module)
        except:
            pprint.pprint(tmp_obj)
            traceback.print_exc()
            raise

def insert_model_objects(endpoint, id_token, model_name, obj_list,
    custom_query=None, mutations_module=None):
    for tmp_obj in obj_list:
        insert_obj(endpoint, id_token, model_name, tmp_obj,
            custom_query=custom_query, mutations_module=mutations_module)

def update_model_objects(endpoint, id_token, model_name,
    obj_list, custom_query=None, mutations_module=None):
    for tmp_obj in obj_list:
        update_obj(endpoint, id_token, model_name, tmp_obj,
            custom_query=custom_query, mutations_module=mutations_module)




'''
Copyright (c) 2020 epiphani, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
'''
import sys
import ecube.gql as cf
import ecube.gql_operations.Subscriptions as S
from future.utils import iteritems
import glob
import yaml 
import os
import signal
import json
from ecube import ecube_hooks as eh
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
import traceback
from jinja2 import Environment, FileSystemLoader
import subprocess
import inspect
import copy 
# Set your users information in the list below
USERS = [
]

# Leave this as an empty list, it is used to automatically
# populate all the usernames from the list above.
USER_LIST = []
USERNAME_DICT = {}
BLACKLISTED_TOKENS = {}
# Leave this as an empty dict, it is used to automatically
# populate all the userid to user objects.
USER_DICT = {}
SLEEP_TIME = 5
# AppSync Endpoint - Subscription Managers Map
APPSYNC_SUB_MGRS_MAP = {}

USE_THREADS = True

# Global know if sandbox connectors should be updated
# or deleted and inserted. Now, default is to update
# if the connectors exist.
UPDATE_CONNECTORS = True

PUBLISH_ONLY = False

# Cutoff size to gzip commands
CUTOFF_SIZE = 10 * 1024
def runCLICMD(cmd, logger):
    out = subprocess.Popen(cmd, shell=True,
               stdout=subprocess.PIPE, 
               stderr=subprocess.STDOUT)
    stdout,stderr = out.communicate()
    logger.log(cf.Logger.DEBUG, "STDOUT: %r" % (stdout))

    if stderr:
        logger.log(cf.Logger.ERROR, "STDERR: %r" % (stderr))

    return str.lstrip(stdout)

def exec_full(filepath, logger):
    ret_val = True
    import os
    global_namespace = {
      "__file__": filepath,
      "__name__": "__main__",
    }

    with open(filepath, 'rb') as file:
        try:
            exec(compile(file.read(), filepath, 'exec'), global_namespace)
        except SystemExit as e:
            logger(cf.Logger.ERROR, "caught exit: code: %r" % (e))
            if str(e) == '-1':
                # TODO: ret_val should be set to False, but if we do that
                # then the meta playbook does not get 'task<ID>_result'
                # for e.g. YaqlEvaluationException: Unable to resolve key ''task2_result'' in expression ''<% ctx().task2_result %>'' from context.
                # So for now, we set it to True will we fix the meta playbook
                ret_val = True
    return ret_val

class Run():
    def __init__(self, args=None):
        global PUBLISH_ONLY
        try:
            tmp_lf = args.log_file
        except:
            tmp_lf = None
        self.logger = cf.Logger(log_to_file=True if tmp_lf else False,
                                log_file=tmp_lf)
        self.logger.set_log_level(cf.Logger.INFO)
        self.args = args 
        self.connector = {}

        try:
            if self.args.publish_playbooks:
                PUBLISH_ONLY = True
        except:
            pass

        if USE_THREADS:
            self.worker_pool = cf.WorkerPool()
        else:
            self.worker_pool = None

    def findAllConnectors(self, d):
        try:
            obj = cf.execute_function_with_retry(cf.get_model_objects,
                (d['endpoint'], d['id_token'], "Connectors", None),
                {}, d['current_env'], d['username'], 1, 0,
                USER_LIST, USER_DICT, USERNAME_DICT, self.logger)
            for val in obj:
                if ((val['name'] in self.connector) and
                    (val['category'] == self.connector[val['name']]['category']) and
                    (val['source'] == self.connector[val['name']]['source'])):
                    if not UPDATE_CONNECTORS:
                        cf.remove_obj(d['endpoint'], d['id_token'],"Connectors", val)
                    else:
                        self.connector[val['name']]['id'] = val['id']
                        cf.update_obj(d['endpoint'], d['id_token'], "Connectors", self.connector[val['name']])
            for k, v in iteritems(self.connector):
                if not 'id' in v:
                    new_conn = cf.insert_obj(d['endpoint'], d['id_token'], 'Connectors', v)
                    v['id'] = new_conn['id']
        except Exception as e:
            tb_output = StringIO()
            traceback.print_exc(file=tb_output)
            o = tb_output.getvalue()
            self.logger.log(cf.Logger.ERROR, o)
            self.logger.log(cf.Logger.ERROR, "findAllConnectors: %r" % (e))

        if PUBLISH_ONLY:
            print("Done Publishing playbooks, exiting...")
            os.kill(os.getpid(), signal.SIGKILL)

        #this should be read from requests 
    def Execute(self):
        ## load the connector from the path
        self.loadConnector()
        
        CURRENT_ENV = self.args.login
        USERS.append({'username': self.args.username, 'passwd': self.args.password})
        cf.gql_main_loop(CURRENT_ENV, self.logger, USERS, BLACKLISTED_TOKENS,
        USER_LIST, USER_DICT, USERNAME_DICT, SLEEP_TIME,
        [
         (S.SUBSCRIPTIONS['oncreateecubesandboxexecution'], self.handle_new_sandbox_execution),
        ], APPSYNC_SUB_MGRS_MAP,
        self.on_error, self.on_sub_error, self.on_connection_error, self.on_close, self.on_subscription_success)

        #this should be read from requests 
    def loadTemplate(self, args):
        template_dir = "%s/files" % (os.path.dirname(inspect.getfile(cf)))
        file_loader = FileSystemLoader(template_dir)
        env = Environment(loader=file_loader)
        template = env.get_template('template.yml')
        cmd = os.path.basename(self.args.name)
        output = template.render(cliName=cmd)
        try:
            f = yaml.safe_load(output)
        except yaml.YAMLError:
            tb_output = StringIO()
            traceback.print_exc(file=tb_output)
            tmp_output = tb_output.getvalue()
            self.logger.log(cf.Logger.ERROR, tmp_output)
        c = get_connector_dict(f, template_dir + "/template.yml", "", self.logger)
        self.connector[c['name']] = c

        return c 

    def isthisforme(self, val):
        for key in self.connector.keys():
            if self.connector[key]["id"] == val["E3Two"]:                
                return True
        return False 

    def handle_cli(self, message, cb_data):
        upd_status = "DONE"
        try:
            val = message['data']['onCreateEcubeSandboxExecution']
            if (not self.isthisforme(val)):
                return 
            data = json.loads(val['E3One'])
            data2 = data['args']['command']
            self.logger.log(cf.Logger.DEBUG, "Got a new execution msg: %r" % (val))
            o = runCLICMD(data2, self.logger)

        except Exception:
            tb_output = StringIO()
            traceback.print_exc(file=tb_output)
            o = tb_output.getvalue()
            self.logger.log(cf.Logger.ERROR, o)
            upd_status = "ERROR"

        try:

            update_dict = {
                'id': val['id'],
                'output': o,
                'status': upd_status
            }
            _ = cf.execute_function_with_retry(cf.update_obj,
                (cb_data['endpoint'], cb_data['id_token'], "EcubeSandboxExecution", update_dict),
                {}, cb_data['current_env'], cb_data['username'], 1, 0,
                USER_LIST, USER_DICT, USERNAME_DICT, self.logger)
        except Exception:
            tb_output = StringIO()
            traceback.print_exc(file=tb_output)
            tmp_output = tb_output.getvalue()
            self.logger.log(cf.Logger.ERROR, tmp_output)


    def ExecuteCLI(self):
        self.logger.log(cf.Logger.DEBUG, "running command with %s" % self.args)
        ## load the connector from the path
        self.loadTemplate(self.args)
        CURRENT_ENV = self.args.login
        USERS.append({'username': self.args.username, 'passwd': self.args.password})
        self.logger.log(cf.Logger.DEBUG, "Waiting for commands...")
        cf.gql_main_loop(CURRENT_ENV, self.logger, USERS, BLACKLISTED_TOKENS,
        USER_LIST, USER_DICT, USERNAME_DICT, SLEEP_TIME,
        [
         (S.SUBSCRIPTIONS['oncreateecubesandboxexecution'], self.handle_cli),
        ], APPSYNC_SUB_MGRS_MAP,
        self.on_error, self.on_sub_error, self.on_connection_error, self.on_close, self.on_subscription_success)


    def loadConnector(self):
        dir_name = self.args.directory
        self.logger.log(cf.Logger.DEBUG, "Reading from: " + dir_name)
        files = get_integration_files(dir_name)
        for tmp_file in files:
            f = parse_file(tmp_file, dir_name, self.logger)
            self.logger.log(cf.Logger.INFO, "read connector %s " %  f)
            self.connector[f['name']] = f
    def on_subscription_success(self, cb_data, sub):
        self.findAllConnectors(cb_data)

    def on_error(self, error, cb_data):
        self.logger.log(cf.Logger.ERROR, "GOT ERROR: %r" % (error))

    def on_sub_error(self, error, cb_data):
        self.logger.log(cf.Logger.ERROR, "GOT SUBSCRIPTION ERROR: %r" % (error))

    def cleanup_sub_mgr(self, cb_data):
        if cb_data['endpoint'] not in BLACKLISTED_TOKENS:
            BLACKLISTED_TOKENS[cb_data['endpoint']] = {}

        BLACKLISTED_TOKENS[cb_data['endpoint']][cb_data['username']] = cb_data['id_token']
        self.logger.log(cf.Logger.INFO, "Blacklisted token: EP: %s U: %s" % (cb_data['endpoint'], cb_data['username']))
        tmp_cb_data = APPSYNC_SUB_MGRS_MAP.pop(cb_data['endpoint'], None)
        tmp_cb_data['manager'].close()

    def on_connection_error(self, error, cb_data):
        self.logger.log(cf.Logger.ERROR, "GOT CONNECTION ERROR: %r" % (error))
        self.cleanup_sub_mgr(cb_data)

    def on_close(self, cb_data):
        self.logger.log(cf.Logger.ERROR, "Run: GOT SOCKET CLOSE")
        #cleanup_sub_mgr(cb_data)

    def handle_new_sandbox_execution(self, message, cb_data):
        if self.worker_pool:
            self.worker_pool.apply_async(self.handle_single_execution,
                (message, cb_data))
        else:
            self.handle_single_execution(message, cb_data)

    def handle_single_execution(self, message, cb_data):
        upd_status = "DONE"
        try:
            val = message['data']['onCreateEcubeSandboxExecution']
            if (not self.isthisforme(val)):
                return 
            self.logger.log(cf.Logger.DEBUG, "Got a new execution msg: %r" % (val))
            data = json.loads(val['E3One'])
            eh.setExecData(data)
            os.environ['ECUBE_HOOKS_PATH'] = os.getcwd() + "/cli"
            exec_full(data['args']['script_path'], self.logger)
            o, rc = eh.getExecOutput()

            if rc != 0:
                upd_status = "ERROR"
        except Exception:
            tb_output = StringIO()
            traceback.print_exc(file=tb_output)
            o = tb_output.getvalue()
            self.logger.log(cf.Logger.ERROR, o)
            upd_status = "ERROR"

        try:
            update_dict = {
                'id': val['id'],
                'output': json.dumps(o, separators=(',', ':')), 
                'status': upd_status
            }

            _ = cf.execute_function_with_retry(cf.update_obj,
                (cb_data['endpoint'], cb_data['id_token'], "EcubeSandboxExecution", update_dict),
                {}, cb_data['current_env'], cb_data['username'], 1, 0,
                USER_LIST, USER_DICT, USERNAME_DICT, self.logger)
        except Exception:
            tb_output = StringIO()
            traceback.print_exc(file=tb_output)
            tmp_output = tb_output.getvalue()
            self.logger.log(cf.Logger.ERROR, tmp_output)

        if self.worker_pool:
            eh.removeThreadData()

def get_files(file_pattern):
    return glob.glob(file_pattern)
# Given a directory, return a list of YML files
# that do not have a corresponding MD file, b/c
# the files that have a MD file are Unreleased
# and are work in progress.
def get_integration_files(dir_name):
    f = get_files(dir_name + "/*.yml")
    return f 
# Parse a given YAML file and store its dictionary in the
# PARSED_CONNECTORS_DICT object
def parse_file(file_name, dir_name, logger):
    logger.log(cf.Logger.log, "Reading FILE: %s DIR: %s" % (file_name, dir_name))
    with open(file_name, 'r') as stream:
        try:
            logger.log(cf.Logger.DEBUG, "Parsing: %s" % (file_name))
            f = yaml.safe_load(stream)
        except yaml.YAMLError:
            tb_output = StringIO()
            traceback.print_exc(file=tb_output)
            tmp_output = tb_output.getvalue()
            logger.log(cf.Logger.ERROR, tmp_output)
        
    c = get_connector_dict(f, file_name, dir_name, logger)
    return c 

INTEGRATION_SOURCE="developer"
INTEGRATION_EPIPHANI_SOURCE="epiphani-multicloud"
INTEGRATION_DIR=""

def get_script_type_extension(file_type):
    if file_type == "python":
        return ".py"
    elif file_type == "javascript":
        return ".js"
    else:
        return None

def get_script_file_name(file_name, file_type, write, logger):
    tmp_f = file_name

    if os.path.basename(file_name).startswith("integration-"):
        tmp_f = file_name.replace("integration-", "")

    extension = get_script_type_extension(file_type)

    if not extension:
        logger.log(cf.Logger.ERROR, "Could not find extension for file NAME: %s TYPE: %s" % (file_name, file_type))
        return None

    tmp_f = tmp_f[:-4] + extension

    if write:
        tmp_f = tmp_f.replace("-", "")

    return tmp_f

# Create a dictionary formatted for the storing in DynamoDB for the parsed file
def get_connector_dict(parsed_file_dict, file_name, dir_name, logger):
    tmp_dict = {}

    try:
        tmp_dict['name'] = parsed_file_dict['name']
        tmp_dict['category'] = parsed_file_dict['category']
        tmp_dict['description'] = parsed_file_dict['description']

        if PUBLISH_ONLY:
            tmp_dict['source'] = INTEGRATION_EPIPHANI_SOURCE
        else:
            tmp_dict['source'] = INTEGRATION_SOURCE

        tmp_dict['md5'] = cf.get_file_md5sum(file_name)
    except Exception as e:
        logger.log(cf.Logger.ERROR, "Missing required param: %r" % (e))
        return None

    if not parsed_file_dict.get('iconPath'):
        tmp_dict['iconPath'] = "/assets/images/logos/epi_connector_icon.png"
    else:
        tmp_dict['iconPath'] = parsed_file_dict.get('iconPath')

    if 'script' in parsed_file_dict:
        if 'type' in parsed_file_dict['script']:
            tmp_dict['scriptType'] = parsed_file_dict['script']['type']

        if 'script' in parsed_file_dict['script']:
            tmp_dict['script'] = parsed_file_dict['script']['script']

            # Script is empty. Try to find and load the script
            script_file_name = get_script_file_name(file_name, tmp_dict['scriptType'], False, logger)

            if not script_file_name:
                logger.log(cf.Logger.ERROR, "Could not parse script file name for: %s" % (file_name))
                sys.exit(-1)

            tmp_dict['scriptPath'] = script_file_name

            # Don't need to put script in DynamoDB
            tmp_dict.pop('script', None)

        if 'commands' in parsed_file_dict['script']:
            tmp_dict['commands'] = json.dumps(parsed_file_dict['script']['commands'], separators=(',', ':'))

            if len(tmp_dict['commands']) > CUTOFF_SIZE:
                tmp_bytes = cf.gzip_string(tmp_dict['commands'])
                tmp_dict['commands'] = cf.b64encode(tmp_bytes)
                tmp_dict['commandsType'] = 'gzip/b64encoded'
            else:
                tmp_dict['commandsType'] = 'plain/text'

    if (('detaileddescription' in parsed_file_dict) and
        (parsed_file_dict['detaileddescription'])):
        tmp_dict['detaileddescription'] = parsed_file_dict['detaileddescription']

    if 'configuration' in parsed_file_dict:
        tmp_dict['configuration'] = json.dumps(parsed_file_dict['configuration'], separators=(',', ':'))

    return tmp_dict




'''
Copyright (c) 2020 epiphani, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
'''
import sys
import time 
import json 
import base64, zlib
from tabulate import tabulate

import ecube.gql as cf

class Playbook(object):
    def __init__(self, args):

        try:
            tmp_lf = args.log_file
        except:
            tmp_lf = None
        self.logger = cf.Logger(log_to_file=True if tmp_lf else False,
                                log_file=tmp_lf)
        self.logger.set_log_level(cf.Logger.INFO)
        self.env = cf.initEnv(args.login, self.logger)
        self.args = args 
    def show(self):
        d = self.env
        try:
            obj = cf.get_model_objects(d['endpoint'], d['id_token'],"AllRunbooks", None)
            extract = {'name', 'id', 'author', 'updatedAt', 'rbVars'}
            arr = []
            for val in obj:
                lst = { key:value for key,value in val.items() if key in extract}
                arr.append(lst)

            print (tabulate(arr, headers="keys"))
        except Exception as e:
            print ("error ", e)
            self.logger.log(cf.Logger.ERROR, "findAddWorkflow: %r" % (e))

    def run(self):
        self.dummy = self.findDummyInv()
        self.findRunPlaybook()

    def findDummyInv(self):
        d = self.env

        try:
            obj = cf.get_model_objects(d['endpoint'], d['id_token'], "Investigation", {'title': {'eq': 'dummy'}, 'from': {'eq': self.args.username}})
            for val in obj:
                if (val['title'] == 'dummy'):
                    INV = val['id']
                    return INV 
            return None 
        except Exception as e:
            self.logger.log(cf.Logger.ERROR, "findAddWorkflow: %r" % (e))

    def findRunPlaybook(self):
        d = self.env
        try:
            obj = cf.get_model_objects(d['endpoint'], d['id_token'], "AllRunbooks", {'name': {'eq': self.args.PBName}})
            for val in obj:
                if (val['name'] == self.args.PBName):
                    return self.runPB(val, d)
            self.logger.log(cf.Logger.ERROR, "PB NOT FOUND %s" % self.args.PBName)

        except Exception as e:
            self.logger.log(cf.Logger.ERROR, "findRunPlaybook: %r" % (e))
    def printOutput(self, out):
        if (self.args.json):
            print(out)
            return 
        oo = json.loads(out)
        for o in oo:
            for k,v in o.items():
                print (k, v)

    def runPB(self, val, d):
        rb = self.ExecutedRunbooksCreate(self.dummy, val['id'], val['name'], d)
        #rb = {'id': "cdadbc86-bb6c-45aa-a328-8064e47944fb"}
        count = 0
        if (self.args.json == False):
            self.logger.log(cf.Logger.DEBUG, "Running Playbook %s ID %s for INV %s" % (val['name'], rb['id'], self.dummy))
        while(True):
            pb = self.findPB(rb['id'], d)
            if (pb == None):
                return None
            if (self.args.json == False):
                print (pb['state'])
            state = pb['state']
            if (state != "new" and state != "processing"):
                op = pb['output']
                if (pb['outputType'] == "gzip/b64encoded"):
                    op = zlib.decompress(base64.b64decode(op))
                self.printOutput(op)
                return state
            count += 1
            if (count > 20):
                print ("Timeout")
                return None
            time.sleep(5)
            
    def ExecutedRunbooksCreate(self, investId, runbookId, title, d):
        author = d['username']
        rbVars = None 
        if (self.args.PBVars):
            rbVars = self.args.PBVars
        cmd = {
            'investigationId': investId, 'author': author,
            'runbookID': runbookId,
            'title': title, 'state': "new",
            'rbVars': rbVars
        }
        try:
            link = cf.insert_obj(d['endpoint'], d['id_token'], "ExecutedRunbooks", cmd)
            return link
        except Exception as e:
            self.logger.log(cf.Logger.ERROR, "Executed RB CREATE : %r" % (e))
            return None

    def findPB(self, pbid, d):
        cmd = {'id': pbid}
        try:
            old = cf.execute_function_with_retry(cf.get_obj,
                                                (d['endpoint'], d['id_token'],
                                                "ExecutedRunbooks", cmd),
                                                {}, d['current_env'], cf.ARTIBOT_USERNAME, 1, 0,
                                                [], {}, {}, self.logger)
            if (old):
                return old
        except Exception as e:
            self.logger.log(cf.Logger.ERROR, "Cannot Find: %r" % (e))
            return None




# This is an auto-generated file, please do not modify

MUTATIONS = {'createallrunbooks': '\n  mutation CreateAllRunbooks($input: CreateAllRunbooksInput!) {\n    createAllRunbooks(input: $input) {\n      id\n      name\n      description\n      content\n      author\n      commands\n      createdAt\n      updatedAt\n      RunBookConnectors\n      type\n      longSave\n      rbVars\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n      connectorForms {\n        nextToken\n      }\n    }\n  }\n',
 'createallworkflows': '\n  mutation CreateAllWorkflows($input: CreateAllWorkflowsInput!) {\n    createAllWorkflows(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      description\n      type\n      author\n      longSave\n      rbVars\n    }\n  }\n',
 'createchannel': '\n  mutation CreateChannel($input: CreateChannelInput!) {\n    createChannel(input: $input) {\n      id\n      messages {\n        nextToken\n      }\n      associated {\n        nextToken\n      }\n      name\n      title\n      members\n      membersState\n      createdAt\n      updatedAt\n      memberLocation\n    }\n  }\n',
 'createchannellink': '\n  mutation CreateChannelLink($input: CreateChannelLinkInput!) {\n    createChannelLink(input: $input) {\n      id\n      contactId\n      channel {\n        id\n        name\n        title\n        members\n        membersState\n        createdAt\n        updatedAt\n        memberLocation\n      }\n      chatId\n      unread\n      createdAt\n      updatedAt\n      user {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n    }\n  }\n',
 'createchat': '\n  mutation CreateChat($input: CreateChatInput!) {\n    createChat(input: $input) {\n      id\n      from\n      content\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createcheatsheet': '\n  mutation CreateCheatSheet($input: CreateCheatSheetInput!) {\n    createCheatSheet(input: $input) {\n      id\n      createdAt\n      updatedAt\n      title\n      author\n      content\n      type\n    }\n  }\n',
 'createchecklist': '\n  mutation CreateCheckList($input: CreateCheckListInput!) {\n    createCheckList(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      description\n      type\n      content\n      author\n      CheckListLink {\n        nextToken\n      }\n      CheckListInvLink {\n        nextToken\n      }\n    }\n  }\n',
 'createchecklistinvlink': '\n  mutation CreateCheckListInvLink($input: CreateCheckListInvLinkInput!) {\n    createCheckListInvLink(input: $input) {\n      id\n      createdAt\n      updatedAt\n      investigationID\n      CheckListID\n      CheckList {\n        id\n        createdAt\n        updatedAt\n        name\n        description\n        type\n        content\n        author\n      }\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n      owner\n    }\n  }\n',
 'createchecklistitem': '\n  mutation CreateCheckListItem($input: CreateCheckListItemInput!) {\n    createCheckListItem(input: $input) {\n      id\n      localID\n      createdAt\n      updatedAt\n      author\n      content\n      title\n      type\n      connection\n      MMNodeConnection\n      investigationID\n      assignee\n      associatedPlaybooks\n      associatedCommands\n      CheckListLink {\n        nextToken\n      }\n    }\n  }\n',
 'createclick': '\n  mutation CreateClick($input: CreateClickInput!) {\n    createClick(input: $input) {\n      id\n      count\n      type\n      featureName\n      createdAt\n      updatedAt\n      readAt\n    }\n  }\n',
 'createclicllink': '\n  mutation CreateCliclLink($input: CreateCLICLLinkInput!) {\n    createCLICLLink(input: $input) {\n      id\n      createdAt\n      updatedAt\n      CheckListItemID\n      lane\n      CheckListID\n      CheckList {\n        id\n        createdAt\n        updatedAt\n        name\n        description\n        type\n        content\n        author\n      }\n      CheckListItem {\n        id\n        localID\n        createdAt\n        updatedAt\n        author\n        content\n        title\n        type\n        connection\n        MMNodeConnection\n        investigationID\n        assignee\n        associatedPlaybooks\n        associatedCommands\n      }\n    }\n  }\n',
 'createcommands': '\n  mutation CreateCommands($input: CreateCommandsInput!) {\n    createCommands(input: $input) {\n      id\n      from\n      mode\n      command\n      output\n      state\n      returnCode\n      investigationId\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'createcomment': '\n  mutation CreateComment($input: CreateCommentInput!) {\n    createComment(input: $input) {\n      id\n      parent\n      createdAt\n      updatedAt\n      author\n      content\n      type\n      links\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'createconferenceconfig': '\n  mutation CreateConferenceConfig($input: CreateConferenceConfigInput!) {\n    createConferenceConfig(input: $input) {\n      id\n      url\n      userName\n      password\n      author\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createconnectorform': '\n  mutation CreateConnectorForm($input: CreateConnectorFormInput!) {\n    createConnectorForm(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      owner\n      connectorID\n      connector {\n        id\n        createdAt\n        updatedAt\n        category\n        name\n        owner\n        description\n        detaileddescription\n        iconPath\n        version\n        md5\n        source\n        configuration\n        commands\n        commandsType\n        script\n        scriptType\n        scriptPath\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n      configData\n      commandData\n      nodeData\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n      runbooks {\n        nextToken\n      }\n    }\n  }\n',
 'createconnectors': '\n  mutation CreateConnectors($input: CreateConnectorsInput!) {\n    createConnectors(input: $input) {\n      id\n      createdAt\n      updatedAt\n      category\n      name\n      owner\n      description\n      detaileddescription\n      iconPath\n      version\n      md5\n      source\n      configuration\n      commands\n      commandsType\n      script\n      scriptType\n      scriptPath\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n    }\n  }\n',
 'createdashboardconfig': '\n  mutation CreateDashboardConfig($input: CreateDashboardConfigInput!) {\n    createDashboardConfig(input: $input) {\n      id\n      url\n      labels {\n        nextToken\n      }\n      author\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createdashboardconfiglabellink': '\n  mutation CreateDashboardConfigLabelLink(\n    $input: CreateDashboardConfigLabelLinkInput!\n  ) {\n    createDashboardConfigLabelLink(input: $input) {\n      id\n      dashboardConfigID\n      labelID\n      dashboardConfig {\n        id\n        url\n        author\n        createdAt\n        updatedAt\n      }\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createecubesandboxexecution': '\n  mutation CreateEcubeSandboxExecution(\n    $input: CreateEcubeSandboxExecutionInput!\n  ) {\n    createEcubeSandboxExecution(input: $input) {\n      id\n      owner\n      output\n      outputType\n      returnCode\n      status\n      createdAt\n      updatedAt\n      E3One\n      E3Two\n      E3Three\n    }\n  }\n',
 'createepicautocomplete': '\n  mutation CreateEpicAutoComplete($input: CreateEpicAutoCompleteInput!) {\n    createEpicAutoComplete(input: $input) {\n      id\n      author\n      cliCommand\n      command\n      createdAt\n      updatedAt\n      investigationId\n      times\n    }\n  }\n',
 'createepiclicommand': '\n  mutation CreateEpicliCommand($input: CreateEPICLICommandInput!) {\n    createEPICLICommand(input: $input) {\n      id\n      createdAt\n      updatedAt\n      author\n      content\n      type\n    }\n  }\n',
 'createexecutedrunbooks': '\n  mutation CreateExecutedRunbooks($input: CreateExecutedRunbooksInput!) {\n    createExecutedRunbooks(input: $input) {\n      id\n      createdAt\n      updatedAt\n      runbookID\n      investigationId\n      output\n      outputType\n      title\n      state\n      author\n      rbVars\n      callbackUrl\n      callbackData\n      source\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n      runbook {\n        id\n        name\n        description\n        content\n        author\n        commands\n        createdAt\n        updatedAt\n        RunBookConnectors\n        type\n        longSave\n        rbVars\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n    }\n  }\n',
 'createexecutedworkflowstate': '\n  mutation CreateExecutedWorkflowState(\n    $input: CreateExecutedWorkflowStateInput!\n  ) {\n    createExecutedWorkflowState(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      state\n      output\n      type\n      allworkflowID\n      investigationID\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'creategroupmember': '\n  mutation CreateGroupMember($input: CreateGroupMemberInput!) {\n    createGroupMember(input: $input) {\n      id\n      hostId\n      groupId\n      host {\n        id\n        inventoryId\n        createdAt\n        updatedAt\n        author\n        host\n        port\n        vars\n      }\n      group {\n        id\n        inventoryId\n        createdAt\n        updatedAt\n        author\n        name\n        groups\n        port\n        vars\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createguardrail': '\n  mutation CreateGuardRail($input: CreateGuardRailInput!) {\n    createGuardRail(input: $input) {\n      id\n      timeOut\n      message\n      labels {\n        nextToken\n      }\n      author\n      active\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createguardraillabellink': '\n  mutation CreateGuardRailLabelLink($input: CreateGuardRailLabelLinkInput!) {\n    createGuardRailLabelLink(input: $input) {\n      id\n      guardRailID\n      labelID\n      guardRail {\n        id\n        timeOut\n        message\n        author\n        active\n        createdAt\n        updatedAt\n      }\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createhint': '\n  mutation CreateHint($input: CreateHintInput!) {\n    createHint(input: $input) {\n      id\n      investigationId\n      createdAt\n      updatedAt\n      type\n      connection\n      pastInvestigationID\n      title\n      importance\n      starrted\n      applied\n      url\n      why\n    }\n  }\n',
 'createinventory': '\n  mutation CreateInventory($input: CreateInventoryInput!) {\n    createInventory(input: $input) {\n      id\n      createdAt\n      updatedAt\n      author\n      name\n      all {\n        nextToken\n      }\n      groups {\n        nextToken\n      }\n      vars\n    }\n  }\n',
 'createinventorygroup': '\n  mutation CreateInventoryGroup($input: CreateInventoryGroupInput!) {\n    createInventoryGroup(input: $input) {\n      id\n      inventoryId\n      createdAt\n      updatedAt\n      author\n      name\n      hosts {\n        nextToken\n      }\n      groups\n      port\n      vars\n      inventory {\n        id\n        createdAt\n        updatedAt\n        author\n        name\n        vars\n      }\n    }\n  }\n',
 'createinventoryhost': '\n  mutation CreateInventoryHost($input: CreateInventoryHostInput!) {\n    createInventoryHost(input: $input) {\n      id\n      inventoryId\n      createdAt\n      updatedAt\n      author\n      host\n      port\n      vars\n      groups {\n        nextToken\n      }\n      inventory {\n        id\n        createdAt\n        updatedAt\n        author\n        name\n        vars\n      }\n    }\n  }\n',
 'createinvestigation': '\n  mutation CreateInvestigation($input: CreateInvestigationInput!) {\n    createInvestigation(input: $input) {\n      id\n      title\n      from\n      createdAt\n      currentState\n      HumanID\n      currentStateID\n      severity\n      priority\n      location\n      labels {\n        nextToken\n      }\n      channelId\n      investigationChannel {\n        id\n        name\n        title\n        members\n        membersState\n        createdAt\n        updatedAt\n        memberLocation\n      }\n      updatedAt\n      resolvedTime\n      resolvedComment\n      Symptoms\n      MMNodeLinks {\n        nextToken\n      }\n      CheckListInvLink {\n        nextToken\n      }\n      meetingType\n      meetingId\n    }\n  }\n',
 'createinvestigationevent': '\n  mutation CreateInvestigationEvent($input: CreateInvestigationEventInput!) {\n    createInvestigationEvent(input: $input) {\n      id\n      createdAt\n      updatedAt\n      author\n      content\n      type\n      links\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'createinvestigationguestuserlink': '\n  mutation CreateInvestigationGuestUserLink(\n    $input: CreateInvestigationGuestUserLinkInput!\n  ) {\n    createInvestigationGuestUserLink(input: $input) {\n      id\n      userName\n      createdAt\n      updatedAt\n      investigationID\n      channelID\n    }\n  }\n',
 'createlabel': '\n  mutation CreateLabel($input: CreateLabelInput!) {\n    createLabel(input: $input) {\n      id\n      name\n      color\n      type\n      investigations {\n        nextToken\n      }\n      userSettings {\n        nextToken\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createlabellink': '\n  mutation CreateLabelLink($input: CreateLabelLinkInput!) {\n    createLabelLink(input: $input) {\n      id\n      investigationID\n      labelID\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'createleadboard': '\n  mutation CreateLeadBoard($input: CreateLeadBoardInput!) {\n    createLeadBoard(input: $input) {\n      id\n      investigationId\n      content\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'createleadboardnode': '\n  mutation CreateLeadBoardNode($input: CreateLeadBoardNodeInput!) {\n    createLeadBoardNode(input: $input) {\n      id\n      localID\n      createdAt\n      updatedAt\n      author\n      content\n      title\n      notes\n      associatedPlaybooks\n      associatedCommands\n      InvestLinks {\n        nextToken\n      }\n    }\n  }\n',
 'createmessage': '\n  mutation CreateMessage($input: CreateMessageInput!) {\n    createMessage(input: $input) {\n      id\n      authorId\n      content\n      type\n      channel {\n        id\n        name\n        title\n        members\n        membersState\n        createdAt\n        updatedAt\n        memberLocation\n      }\n      messageChannelId\n      createdAt\n      updatedAt\n      author {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n    }\n  }\n',
 'createmmnodelink': '\n  mutation CreateMmNodeLink($input: CreateMMNodeLinkInput!) {\n    createMMNodeLink(input: $input) {\n      id\n      createdAt\n      updatedAt\n      assigned\n      state\n      content\n      author\n      investigationID\n      connectionToTask\n      LBNID\n      LeadBoardNodeId {\n        id\n        localID\n        createdAt\n        updatedAt\n        author\n        content\n        title\n        notes\n        associatedPlaybooks\n        associatedCommands\n      }\n      investigationId {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'createnotification': '\n  mutation CreateNotification($input: CreateNotificationInput!) {\n    createNotification(input: $input) {\n      id\n      userName\n      content\n      containerType\n      containerId\n      investigationId\n      isViewed\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createpin': '\n  mutation CreatePin($input: CreatePinInput!) {\n    createPin(input: $input) {\n      id\n      imageId\n      content\n      createdAt\n      updatedAt\n      author\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'createprivatekey': '\n  mutation CreatePrivateKey($input: CreatePrivateKeyInput!) {\n    createPrivateKey(input: $input) {\n      id\n      sshKey\n      userName\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createremotecontrol': '\n  mutation CreateRemoteControl($input: CreateRemoteControlInput!) {\n    createRemoteControl(input: $input) {\n      id\n      from\n      to\n      command\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createreportsummaryeditor': '\n  mutation CreateReportSummaryEditor($input: CreateReportSummaryEditorInput!) {\n    createReportSummaryEditor(input: $input) {\n      id\n      investigationId\n      createdAt\n      updatedAt\n      author\n      content\n      type\n    }\n  }\n',
 'createrunbookconnectorformlink': '\n  mutation CreateRunbookConnectorFormLink(\n    $input: CreateRunbookConnectorFormLinkInput!\n  ) {\n    createRunbookConnectorFormLink(input: $input) {\n      id\n      connectorFormId\n      runbookId\n      connectorForm {\n        id\n        createdAt\n        updatedAt\n        name\n        owner\n        connectorID\n        configData\n        commandData\n        nodeData\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n      createdAt\n      updatedAt\n      runbook {\n        id\n        name\n        description\n        content\n        author\n        commands\n        createdAt\n        updatedAt\n        RunBookConnectors\n        type\n        longSave\n        rbVars\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n    }\n  }\n',
 'createschedpbs': '\n  mutation CreateSchedPBs($input: CreateSchedPBsInput!) {\n    createSchedPBs(input: $input) {\n      id\n      pbId\n      frequency\n      pbName\n      rbVars\n      scheduled\n      createdAt\n      updatedAt\n      author\n    }\n  }\n',
 'createslackchannelinfo': '\n  mutation CreateSlackChannelInfo($input: CreateSlackChannelInfoInput!) {\n    createSlackChannelInfo(input: $input) {\n      id\n      owner\n      slackChannelId\n      slackAppId\n      slackTeamId\n      createdAt\n      updatedAt\n      slackTeam {\n        id\n        owner\n        teamId\n        teamName\n        slackAppId\n        botId\n        botToken\n        botScope\n        enterprise\n        createdAt\n        updatedAt\n      }\n      usersEnabled {\n        nextToken\n      }\n    }\n  }\n',
 'createslackenabledchannelsuserlink': '\n  mutation CreateSlackEnabledChannelsUserLink(\n    $input: CreateSlackEnabledChannelsUserLinkInput!\n  ) {\n    createSlackEnabledChannelsUserLink(input: $input) {\n      id\n      owner\n      slackUserId\n      slackChannelId\n      createdAt\n      updatedAt\n      slackUser {\n        id\n        userAppSyncId\n        userName\n        userId\n        userScope\n        userToken\n        slackAppId\n        slackTeamId\n        createdAt\n        updatedAt\n      }\n      slackChannel {\n        id\n        owner\n        slackChannelId\n        slackAppId\n        slackTeamId\n        createdAt\n        updatedAt\n      }\n    }\n  }\n',
 'createslackteaminfo': '\n  mutation CreateSlackTeamInfo($input: CreateSlackTeamInfoInput!) {\n    createSlackTeamInfo(input: $input) {\n      id\n      owner\n      teamId\n      teamName\n      slackAppId\n      botId\n      botToken\n      botScope\n      enterprise\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createslackuserinfo': '\n  mutation CreateSlackUserInfo($input: CreateSlackUserInfoInput!) {\n    createSlackUserInfo(input: $input) {\n      id\n      userAppSyncId\n      userName\n      userId\n      userScope\n      userToken\n      slackAppId\n      slackTeamId\n      createdAt\n      updatedAt\n      userGqlInfo {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n      slackTeam {\n        id\n        owner\n        teamId\n        teamName\n        slackAppId\n        botId\n        botToken\n        botScope\n        enterprise\n        createdAt\n        updatedAt\n      }\n      channelsEnabled {\n        nextToken\n      }\n    }\n  }\n',
 'createsshtabs': '\n  mutation CreateSshTabs($input: CreateSshTabsInput!) {\n    createSshTabs(input: $input) {\n      id\n      investigationId\n      host\n      author\n      userName\n      createdAt\n      updatedAt\n    }\n  }\n',
 'createstash': '\n  mutation CreateStash($input: CreateStashInput!) {\n    createStash(input: $input) {\n      id\n      Key\n      Value\n      createdAt\n      updatedAt\n      author\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n    }\n  }\n',
 'createstate': '\n  mutation CreateState($input: CreateStateInput!) {\n    createState(input: $input) {\n      id\n      name\n      createdAt\n      updatedAt\n      author\n      lastState\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'createsummaryinvestigation': '\n  mutation CreateSummaryInvestigation(\n    $input: CreateSummaryInvestigationInput!\n  ) {\n    createSummaryInvestigation(input: $input) {\n      id\n      createdAt\n      updatedAt\n      resolvedTime\n      OriginalID\n      title\n      HumanID\n      Commands\n      InvestigationEvents\n      CompleteInvestigationEvents\n      Participants\n      Comments\n      Pins\n      AvailableRunbooks\n      UsedRunbooks\n      Hints\n      Symptoms\n      owner\n    }\n  }\n',
 'createtenantcustomer': '\n  mutation CreateTenantCustomer($input: CreateTenantCustomerInput!) {\n    createTenantCustomer(input: $input) {\n      label\n      impactLabels\n      createdAt\n      updatedAt\n      customerName\n      email\n      phoneno\n      SLAType\n      city\n      state\n      contry\n      zipcode\n    }\n  }\n',
 'createtranscriptionrecords': '\n  mutation CreateTranscriptionRecords(\n    $input: CreateTranscriptionRecordsInput!\n  ) {\n    createTranscriptionRecords(input: $input) {\n      id\n      investigationId\n      createdAt\n      updatedAt\n      spokenBy\n      content\n    }\n  }\n',
 'createusedtogenhintalready': '\n  mutation CreateUsedToGenHintAlready(\n    $input: CreateUsedToGenHintAlreadyInput!\n  ) {\n    createUsedToGenHintAlready(input: $input) {\n      id\n      createdAt\n      updatedAt\n      investigationId\n      commands\n      text\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'createuser': '\n  mutation CreateUser($input: CreateUserInput!) {\n    createUser(input: $input) {\n      id\n      username\n      email\n      firstname\n      lastname\n      avatar\n      mood\n      name\n      role\n      status\n      timezone\n      phone\n      address\n      location\n      chatList {\n        nextToken\n      }\n      messages {\n        nextToken\n      }\n      createdAt\n      updatedAt\n      owner\n    }\n  }\n',
 'createuseractivity': '\n  mutation CreateUserActivity($input: CreateUserActivityInput!) {\n    createUserActivity(input: $input) {\n      id\n      userID\n      status\n      investigationID\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n      user {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n    }\n  }\n',
 'createusersetting': '\n  mutation CreateUserSetting($input: CreateUserSettingInput!) {\n    createUserSetting(input: $input) {\n      id\n      userId\n      owner\n      createdAt\n      updatedAt\n      labels {\n        nextToken\n      }\n      value\n      url\n      name\n      type\n      showWelcomeNext\n      showWelcomeSkip\n      showWelcomeRemind\n      welcomeData\n    }\n  }\n',
 'createusersettinglabellink': '\n  mutation CreateUserSettingLabelLink(\n    $input: CreateUserSettingLabelLinkInput!\n  ) {\n    createUserSettingLabelLink(input: $input) {\n      id\n      userSettingId\n      userSetting {\n        id\n        userId\n        owner\n        createdAt\n        updatedAt\n        value\n        url\n        name\n        type\n        showWelcomeNext\n        showWelcomeSkip\n        showWelcomeRemind\n        welcomeData\n      }\n      labelID\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      type\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deleteallrunbooks': '\n  mutation DeleteAllRunbooks($input: DeleteAllRunbooksInput!) {\n    deleteAllRunbooks(input: $input) {\n      id\n      name\n      description\n      content\n      author\n      commands\n      createdAt\n      updatedAt\n      RunBookConnectors\n      type\n      longSave\n      rbVars\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n      connectorForms {\n        nextToken\n      }\n    }\n  }\n',
 'deleteallworkflows': '\n  mutation DeleteAllWorkflows($input: DeleteAllWorkflowsInput!) {\n    deleteAllWorkflows(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      description\n      type\n      author\n      longSave\n      rbVars\n    }\n  }\n',
 'deletechannel': '\n  mutation DeleteChannel($input: DeleteChannelInput!) {\n    deleteChannel(input: $input) {\n      id\n      messages {\n        nextToken\n      }\n      associated {\n        nextToken\n      }\n      name\n      title\n      members\n      membersState\n      createdAt\n      updatedAt\n      memberLocation\n    }\n  }\n',
 'deletechannellink': '\n  mutation DeleteChannelLink($input: DeleteChannelLinkInput!) {\n    deleteChannelLink(input: $input) {\n      id\n      contactId\n      channel {\n        id\n        name\n        title\n        members\n        membersState\n        createdAt\n        updatedAt\n        memberLocation\n      }\n      chatId\n      unread\n      createdAt\n      updatedAt\n      user {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n    }\n  }\n',
 'deletechat': '\n  mutation DeleteChat($input: DeleteChatInput!) {\n    deleteChat(input: $input) {\n      id\n      from\n      content\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deletecheatsheet': '\n  mutation DeleteCheatSheet($input: DeleteCheatSheetInput!) {\n    deleteCheatSheet(input: $input) {\n      id\n      createdAt\n      updatedAt\n      title\n      author\n      content\n      type\n    }\n  }\n',
 'deletechecklist': '\n  mutation DeleteCheckList($input: DeleteCheckListInput!) {\n    deleteCheckList(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      description\n      type\n      content\n      author\n      CheckListLink {\n        nextToken\n      }\n      CheckListInvLink {\n        nextToken\n      }\n    }\n  }\n',
 'deletechecklistinvlink': '\n  mutation DeleteCheckListInvLink($input: DeleteCheckListInvLinkInput!) {\n    deleteCheckListInvLink(input: $input) {\n      id\n      createdAt\n      updatedAt\n      investigationID\n      CheckListID\n      CheckList {\n        id\n        createdAt\n        updatedAt\n        name\n        description\n        type\n        content\n        author\n      }\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n      owner\n    }\n  }\n',
 'deletechecklistitem': '\n  mutation DeleteCheckListItem($input: DeleteCheckListItemInput!) {\n    deleteCheckListItem(input: $input) {\n      id\n      localID\n      createdAt\n      updatedAt\n      author\n      content\n      title\n      type\n      connection\n      MMNodeConnection\n      investigationID\n      assignee\n      associatedPlaybooks\n      associatedCommands\n      CheckListLink {\n        nextToken\n      }\n    }\n  }\n',
 'deleteclick': '\n  mutation DeleteClick($input: DeleteClickInput!) {\n    deleteClick(input: $input) {\n      id\n      count\n      type\n      featureName\n      createdAt\n      updatedAt\n      readAt\n    }\n  }\n',
 'deleteclicllink': '\n  mutation DeleteCliclLink($input: DeleteCLICLLinkInput!) {\n    deleteCLICLLink(input: $input) {\n      id\n      createdAt\n      updatedAt\n      CheckListItemID\n      lane\n      CheckListID\n      CheckList {\n        id\n        createdAt\n        updatedAt\n        name\n        description\n        type\n        content\n        author\n      }\n      CheckListItem {\n        id\n        localID\n        createdAt\n        updatedAt\n        author\n        content\n        title\n        type\n        connection\n        MMNodeConnection\n        investigationID\n        assignee\n        associatedPlaybooks\n        associatedCommands\n      }\n    }\n  }\n',
 'deletecommands': '\n  mutation DeleteCommands($input: DeleteCommandsInput!) {\n    deleteCommands(input: $input) {\n      id\n      from\n      mode\n      command\n      output\n      state\n      returnCode\n      investigationId\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'deletecomment': '\n  mutation DeleteComment($input: DeleteCommentInput!) {\n    deleteComment(input: $input) {\n      id\n      parent\n      createdAt\n      updatedAt\n      author\n      content\n      type\n      links\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'deleteconferenceconfig': '\n  mutation DeleteConferenceConfig($input: DeleteConferenceConfigInput!) {\n    deleteConferenceConfig(input: $input) {\n      id\n      url\n      userName\n      password\n      author\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deleteconnectorform': '\n  mutation DeleteConnectorForm($input: DeleteConnectorFormInput!) {\n    deleteConnectorForm(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      owner\n      connectorID\n      connector {\n        id\n        createdAt\n        updatedAt\n        category\n        name\n        owner\n        description\n        detaileddescription\n        iconPath\n        version\n        md5\n        source\n        configuration\n        commands\n        commandsType\n        script\n        scriptType\n        scriptPath\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n      configData\n      commandData\n      nodeData\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n      runbooks {\n        nextToken\n      }\n    }\n  }\n',
 'deleteconnectors': '\n  mutation DeleteConnectors($input: DeleteConnectorsInput!) {\n    deleteConnectors(input: $input) {\n      id\n      createdAt\n      updatedAt\n      category\n      name\n      owner\n      description\n      detaileddescription\n      iconPath\n      version\n      md5\n      source\n      configuration\n      commands\n      commandsType\n      script\n      scriptType\n      scriptPath\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n    }\n  }\n',
 'deletedashboardconfig': '\n  mutation DeleteDashboardConfig($input: DeleteDashboardConfigInput!) {\n    deleteDashboardConfig(input: $input) {\n      id\n      url\n      labels {\n        nextToken\n      }\n      author\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deletedashboardconfiglabellink': '\n  mutation DeleteDashboardConfigLabelLink(\n    $input: DeleteDashboardConfigLabelLinkInput!\n  ) {\n    deleteDashboardConfigLabelLink(input: $input) {\n      id\n      dashboardConfigID\n      labelID\n      dashboardConfig {\n        id\n        url\n        author\n        createdAt\n        updatedAt\n      }\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deleteecubesandboxexecution': '\n  mutation DeleteEcubeSandboxExecution(\n    $input: DeleteEcubeSandboxExecutionInput!\n  ) {\n    deleteEcubeSandboxExecution(input: $input) {\n      id\n      owner\n      output\n      outputType\n      returnCode\n      status\n      createdAt\n      updatedAt\n      E3One\n      E3Two\n      E3Three\n    }\n  }\n',
 'deleteepicautocomplete': '\n  mutation DeleteEpicAutoComplete($input: DeleteEpicAutoCompleteInput!) {\n    deleteEpicAutoComplete(input: $input) {\n      id\n      author\n      cliCommand\n      command\n      createdAt\n      updatedAt\n      investigationId\n      times\n    }\n  }\n',
 'deleteepiclicommand': '\n  mutation DeleteEpicliCommand($input: DeleteEPICLICommandInput!) {\n    deleteEPICLICommand(input: $input) {\n      id\n      createdAt\n      updatedAt\n      author\n      content\n      type\n    }\n  }\n',
 'deleteexecutedrunbooks': '\n  mutation DeleteExecutedRunbooks($input: DeleteExecutedRunbooksInput!) {\n    deleteExecutedRunbooks(input: $input) {\n      id\n      createdAt\n      updatedAt\n      runbookID\n      investigationId\n      output\n      outputType\n      title\n      state\n      author\n      rbVars\n      callbackUrl\n      callbackData\n      source\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n      runbook {\n        id\n        name\n        description\n        content\n        author\n        commands\n        createdAt\n        updatedAt\n        RunBookConnectors\n        type\n        longSave\n        rbVars\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n    }\n  }\n',
 'deleteexecutedworkflowstate': '\n  mutation DeleteExecutedWorkflowState(\n    $input: DeleteExecutedWorkflowStateInput!\n  ) {\n    deleteExecutedWorkflowState(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      state\n      output\n      type\n      allworkflowID\n      investigationID\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'deletegroupmember': '\n  mutation DeleteGroupMember($input: DeleteGroupMemberInput!) {\n    deleteGroupMember(input: $input) {\n      id\n      hostId\n      groupId\n      host {\n        id\n        inventoryId\n        createdAt\n        updatedAt\n        author\n        host\n        port\n        vars\n      }\n      group {\n        id\n        inventoryId\n        createdAt\n        updatedAt\n        author\n        name\n        groups\n        port\n        vars\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deleteguardrail': '\n  mutation DeleteGuardRail($input: DeleteGuardRailInput!) {\n    deleteGuardRail(input: $input) {\n      id\n      timeOut\n      message\n      labels {\n        nextToken\n      }\n      author\n      active\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deleteguardraillabellink': '\n  mutation DeleteGuardRailLabelLink($input: DeleteGuardRailLabelLinkInput!) {\n    deleteGuardRailLabelLink(input: $input) {\n      id\n      guardRailID\n      labelID\n      guardRail {\n        id\n        timeOut\n        message\n        author\n        active\n        createdAt\n        updatedAt\n      }\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deletehint': '\n  mutation DeleteHint($input: DeleteHintInput!) {\n    deleteHint(input: $input) {\n      id\n      investigationId\n      createdAt\n      updatedAt\n      type\n      connection\n      pastInvestigationID\n      title\n      importance\n      starrted\n      applied\n      url\n      why\n    }\n  }\n',
 'deleteinventory': '\n  mutation DeleteInventory($input: DeleteInventoryInput!) {\n    deleteInventory(input: $input) {\n      id\n      createdAt\n      updatedAt\n      author\n      name\n      all {\n        nextToken\n      }\n      groups {\n        nextToken\n      }\n      vars\n    }\n  }\n',
 'deleteinventorygroup': '\n  mutation DeleteInventoryGroup($input: DeleteInventoryGroupInput!) {\n    deleteInventoryGroup(input: $input) {\n      id\n      inventoryId\n      createdAt\n      updatedAt\n      author\n      name\n      hosts {\n        nextToken\n      }\n      groups\n      port\n      vars\n      inventory {\n        id\n        createdAt\n        updatedAt\n        author\n        name\n        vars\n      }\n    }\n  }\n',
 'deleteinventoryhost': '\n  mutation DeleteInventoryHost($input: DeleteInventoryHostInput!) {\n    deleteInventoryHost(input: $input) {\n      id\n      inventoryId\n      createdAt\n      updatedAt\n      author\n      host\n      port\n      vars\n      groups {\n        nextToken\n      }\n      inventory {\n        id\n        createdAt\n        updatedAt\n        author\n        name\n        vars\n      }\n    }\n  }\n',
 'deleteinvestigation': '\n  mutation DeleteInvestigation($input: DeleteInvestigationInput!) {\n    deleteInvestigation(input: $input) {\n      id\n      title\n      from\n      createdAt\n      currentState\n      HumanID\n      currentStateID\n      severity\n      priority\n      location\n      labels {\n        nextToken\n      }\n      channelId\n      investigationChannel {\n        id\n        name\n        title\n        members\n        membersState\n        createdAt\n        updatedAt\n        memberLocation\n      }\n      updatedAt\n      resolvedTime\n      resolvedComment\n      Symptoms\n      MMNodeLinks {\n        nextToken\n      }\n      CheckListInvLink {\n        nextToken\n      }\n      meetingType\n      meetingId\n    }\n  }\n',
 'deleteinvestigationevent': '\n  mutation DeleteInvestigationEvent($input: DeleteInvestigationEventInput!) {\n    deleteInvestigationEvent(input: $input) {\n      id\n      createdAt\n      updatedAt\n      author\n      content\n      type\n      links\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'deleteinvestigationguestuserlink': '\n  mutation DeleteInvestigationGuestUserLink(\n    $input: DeleteInvestigationGuestUserLinkInput!\n  ) {\n    deleteInvestigationGuestUserLink(input: $input) {\n      id\n      userName\n      createdAt\n      updatedAt\n      investigationID\n      channelID\n    }\n  }\n',
 'deletelabel': '\n  mutation DeleteLabel($input: DeleteLabelInput!) {\n    deleteLabel(input: $input) {\n      id\n      name\n      color\n      type\n      investigations {\n        nextToken\n      }\n      userSettings {\n        nextToken\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deletelabellink': '\n  mutation DeleteLabelLink($input: DeleteLabelLinkInput!) {\n    deleteLabelLink(input: $input) {\n      id\n      investigationID\n      labelID\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'deleteleadboard': '\n  mutation DeleteLeadBoard($input: DeleteLeadBoardInput!) {\n    deleteLeadBoard(input: $input) {\n      id\n      investigationId\n      content\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'deleteleadboardnode': '\n  mutation DeleteLeadBoardNode($input: DeleteLeadBoardNodeInput!) {\n    deleteLeadBoardNode(input: $input) {\n      id\n      localID\n      createdAt\n      updatedAt\n      author\n      content\n      title\n      notes\n      associatedPlaybooks\n      associatedCommands\n      InvestLinks {\n        nextToken\n      }\n    }\n  }\n',
 'deletemessage': '\n  mutation DeleteMessage($input: DeleteMessageInput!) {\n    deleteMessage(input: $input) {\n      id\n      authorId\n      content\n      type\n      channel {\n        id\n        name\n        title\n        members\n        membersState\n        createdAt\n        updatedAt\n        memberLocation\n      }\n      messageChannelId\n      createdAt\n      updatedAt\n      author {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n    }\n  }\n',
 'deletemmnodelink': '\n  mutation DeleteMmNodeLink($input: DeleteMMNodeLinkInput!) {\n    deleteMMNodeLink(input: $input) {\n      id\n      createdAt\n      updatedAt\n      assigned\n      state\n      content\n      author\n      investigationID\n      connectionToTask\n      LBNID\n      LeadBoardNodeId {\n        id\n        localID\n        createdAt\n        updatedAt\n        author\n        content\n        title\n        notes\n        associatedPlaybooks\n        associatedCommands\n      }\n      investigationId {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'deletenotification': '\n  mutation DeleteNotification($input: DeleteNotificationInput!) {\n    deleteNotification(input: $input) {\n      id\n      userName\n      content\n      containerType\n      containerId\n      investigationId\n      isViewed\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deletepin': '\n  mutation DeletePin($input: DeletePinInput!) {\n    deletePin(input: $input) {\n      id\n      imageId\n      content\n      createdAt\n      updatedAt\n      author\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'deleteprivatekey': '\n  mutation DeletePrivateKey($input: DeletePrivateKeyInput!) {\n    deletePrivateKey(input: $input) {\n      id\n      sshKey\n      userName\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deleteremotecontrol': '\n  mutation DeleteRemoteControl($input: DeleteRemoteControlInput!) {\n    deleteRemoteControl(input: $input) {\n      id\n      from\n      to\n      command\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deletereportsummaryeditor': '\n  mutation DeleteReportSummaryEditor($input: DeleteReportSummaryEditorInput!) {\n    deleteReportSummaryEditor(input: $input) {\n      id\n      investigationId\n      createdAt\n      updatedAt\n      author\n      content\n      type\n    }\n  }\n',
 'deleterunbookconnectorformlink': '\n  mutation DeleteRunbookConnectorFormLink(\n    $input: DeleteRunbookConnectorFormLinkInput!\n  ) {\n    deleteRunbookConnectorFormLink(input: $input) {\n      id\n      connectorFormId\n      runbookId\n      connectorForm {\n        id\n        createdAt\n        updatedAt\n        name\n        owner\n        connectorID\n        configData\n        commandData\n        nodeData\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n      createdAt\n      updatedAt\n      runbook {\n        id\n        name\n        description\n        content\n        author\n        commands\n        createdAt\n        updatedAt\n        RunBookConnectors\n        type\n        longSave\n        rbVars\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n    }\n  }\n',
 'deleteschedpbs': '\n  mutation DeleteSchedPBs($input: DeleteSchedPBsInput!) {\n    deleteSchedPBs(input: $input) {\n      id\n      pbId\n      frequency\n      pbName\n      rbVars\n      scheduled\n      createdAt\n      updatedAt\n      author\n    }\n  }\n',
 'deleteslackchannelinfo': '\n  mutation DeleteSlackChannelInfo($input: DeleteSlackChannelInfoInput!) {\n    deleteSlackChannelInfo(input: $input) {\n      id\n      owner\n      slackChannelId\n      slackAppId\n      slackTeamId\n      createdAt\n      updatedAt\n      slackTeam {\n        id\n        owner\n        teamId\n        teamName\n        slackAppId\n        botId\n        botToken\n        botScope\n        enterprise\n        createdAt\n        updatedAt\n      }\n      usersEnabled {\n        nextToken\n      }\n    }\n  }\n',
 'deleteslackenabledchannelsuserlink': '\n  mutation DeleteSlackEnabledChannelsUserLink(\n    $input: DeleteSlackEnabledChannelsUserLinkInput!\n  ) {\n    deleteSlackEnabledChannelsUserLink(input: $input) {\n      id\n      owner\n      slackUserId\n      slackChannelId\n      createdAt\n      updatedAt\n      slackUser {\n        id\n        userAppSyncId\n        userName\n        userId\n        userScope\n        userToken\n        slackAppId\n        slackTeamId\n        createdAt\n        updatedAt\n      }\n      slackChannel {\n        id\n        owner\n        slackChannelId\n        slackAppId\n        slackTeamId\n        createdAt\n        updatedAt\n      }\n    }\n  }\n',
 'deleteslackteaminfo': '\n  mutation DeleteSlackTeamInfo($input: DeleteSlackTeamInfoInput!) {\n    deleteSlackTeamInfo(input: $input) {\n      id\n      owner\n      teamId\n      teamName\n      slackAppId\n      botId\n      botToken\n      botScope\n      enterprise\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deleteslackuserinfo': '\n  mutation DeleteSlackUserInfo($input: DeleteSlackUserInfoInput!) {\n    deleteSlackUserInfo(input: $input) {\n      id\n      userAppSyncId\n      userName\n      userId\n      userScope\n      userToken\n      slackAppId\n      slackTeamId\n      createdAt\n      updatedAt\n      userGqlInfo {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n      slackTeam {\n        id\n        owner\n        teamId\n        teamName\n        slackAppId\n        botId\n        botToken\n        botScope\n        enterprise\n        createdAt\n        updatedAt\n      }\n      channelsEnabled {\n        nextToken\n      }\n    }\n  }\n',
 'deletesshtabs': '\n  mutation DeleteSshTabs($input: DeleteSshTabsInput!) {\n    deleteSshTabs(input: $input) {\n      id\n      investigationId\n      host\n      author\n      userName\n      createdAt\n      updatedAt\n    }\n  }\n',
 'deletestash': '\n  mutation DeleteStash($input: DeleteStashInput!) {\n    deleteStash(input: $input) {\n      id\n      Key\n      Value\n      createdAt\n      updatedAt\n      author\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n    }\n  }\n',
 'deletestate': '\n  mutation DeleteState($input: DeleteStateInput!) {\n    deleteState(input: $input) {\n      id\n      name\n      createdAt\n      updatedAt\n      author\n      lastState\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'deletesummaryinvestigation': '\n  mutation DeleteSummaryInvestigation(\n    $input: DeleteSummaryInvestigationInput!\n  ) {\n    deleteSummaryInvestigation(input: $input) {\n      id\n      createdAt\n      updatedAt\n      resolvedTime\n      OriginalID\n      title\n      HumanID\n      Commands\n      InvestigationEvents\n      CompleteInvestigationEvents\n      Participants\n      Comments\n      Pins\n      AvailableRunbooks\n      UsedRunbooks\n      Hints\n      Symptoms\n      owner\n    }\n  }\n',
 'deletetenantcustomer': '\n  mutation DeleteTenantCustomer($input: DeleteTenantCustomerInput!) {\n    deleteTenantCustomer(input: $input) {\n      label\n      impactLabels\n      createdAt\n      updatedAt\n      customerName\n      email\n      phoneno\n      SLAType\n      city\n      state\n      contry\n      zipcode\n    }\n  }\n',
 'deletetranscriptionrecords': '\n  mutation DeleteTranscriptionRecords(\n    $input: DeleteTranscriptionRecordsInput!\n  ) {\n    deleteTranscriptionRecords(input: $input) {\n      id\n      investigationId\n      createdAt\n      updatedAt\n      spokenBy\n      content\n    }\n  }\n',
 'deleteusedtogenhintalready': '\n  mutation DeleteUsedToGenHintAlready(\n    $input: DeleteUsedToGenHintAlreadyInput!\n  ) {\n    deleteUsedToGenHintAlready(input: $input) {\n      id\n      createdAt\n      updatedAt\n      investigationId\n      commands\n      text\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'deleteuser': '\n  mutation DeleteUser($input: DeleteUserInput!) {\n    deleteUser(input: $input) {\n      id\n      username\n      email\n      firstname\n      lastname\n      avatar\n      mood\n      name\n      role\n      status\n      timezone\n      phone\n      address\n      location\n      chatList {\n        nextToken\n      }\n      messages {\n        nextToken\n      }\n      createdAt\n      updatedAt\n      owner\n    }\n  }\n',
 'deleteuseractivity': '\n  mutation DeleteUserActivity($input: DeleteUserActivityInput!) {\n    deleteUserActivity(input: $input) {\n      id\n      userID\n      status\n      investigationID\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n      user {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n    }\n  }\n',
 'deleteusersetting': '\n  mutation DeleteUserSetting($input: DeleteUserSettingInput!) {\n    deleteUserSetting(input: $input) {\n      id\n      userId\n      owner\n      createdAt\n      updatedAt\n      labels {\n        nextToken\n      }\n      value\n      url\n      name\n      type\n      showWelcomeNext\n      showWelcomeSkip\n      showWelcomeRemind\n      welcomeData\n    }\n  }\n',
 'deleteusersettinglabellink': '\n  mutation DeleteUserSettingLabelLink(\n    $input: DeleteUserSettingLabelLinkInput!\n  ) {\n    deleteUserSettingLabelLink(input: $input) {\n      id\n      userSettingId\n      userSetting {\n        id\n        userId\n        owner\n        createdAt\n        updatedAt\n        value\n        url\n        name\n        type\n        showWelcomeNext\n        showWelcomeSkip\n        showWelcomeRemind\n        welcomeData\n      }\n      labelID\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      type\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updateallrunbooks': '\n  mutation UpdateAllRunbooks($input: UpdateAllRunbooksInput!) {\n    updateAllRunbooks(input: $input) {\n      id\n      name\n      description\n      content\n      author\n      commands\n      createdAt\n      updatedAt\n      RunBookConnectors\n      type\n      longSave\n      rbVars\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n      connectorForms {\n        nextToken\n      }\n    }\n  }\n',
 'updateallworkflows': '\n  mutation UpdateAllWorkflows($input: UpdateAllWorkflowsInput!) {\n    updateAllWorkflows(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      description\n      type\n      author\n      longSave\n      rbVars\n    }\n  }\n',
 'updatechannel': '\n  mutation UpdateChannel($input: UpdateChannelInput!) {\n    updateChannel(input: $input) {\n      id\n      messages {\n        nextToken\n      }\n      associated {\n        nextToken\n      }\n      name\n      title\n      members\n      membersState\n      createdAt\n      updatedAt\n      memberLocation\n    }\n  }\n',
 'updatechannellink': '\n  mutation UpdateChannelLink($input: UpdateChannelLinkInput!) {\n    updateChannelLink(input: $input) {\n      id\n      contactId\n      channel {\n        id\n        name\n        title\n        members\n        membersState\n        createdAt\n        updatedAt\n        memberLocation\n      }\n      chatId\n      unread\n      createdAt\n      updatedAt\n      user {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n    }\n  }\n',
 'updatechat': '\n  mutation UpdateChat($input: UpdateChatInput!) {\n    updateChat(input: $input) {\n      id\n      from\n      content\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updatecheatsheet': '\n  mutation UpdateCheatSheet($input: UpdateCheatSheetInput!) {\n    updateCheatSheet(input: $input) {\n      id\n      createdAt\n      updatedAt\n      title\n      author\n      content\n      type\n    }\n  }\n',
 'updatechecklist': '\n  mutation UpdateCheckList($input: UpdateCheckListInput!) {\n    updateCheckList(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      description\n      type\n      content\n      author\n      CheckListLink {\n        nextToken\n      }\n      CheckListInvLink {\n        nextToken\n      }\n    }\n  }\n',
 'updatechecklistinvlink': '\n  mutation UpdateCheckListInvLink($input: UpdateCheckListInvLinkInput!) {\n    updateCheckListInvLink(input: $input) {\n      id\n      createdAt\n      updatedAt\n      investigationID\n      CheckListID\n      CheckList {\n        id\n        createdAt\n        updatedAt\n        name\n        description\n        type\n        content\n        author\n      }\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n      owner\n    }\n  }\n',
 'updatechecklistitem': '\n  mutation UpdateCheckListItem($input: UpdateCheckListItemInput!) {\n    updateCheckListItem(input: $input) {\n      id\n      localID\n      createdAt\n      updatedAt\n      author\n      content\n      title\n      type\n      connection\n      MMNodeConnection\n      investigationID\n      assignee\n      associatedPlaybooks\n      associatedCommands\n      CheckListLink {\n        nextToken\n      }\n    }\n  }\n',
 'updateclick': '\n  mutation UpdateClick($input: UpdateClickInput!) {\n    updateClick(input: $input) {\n      id\n      count\n      type\n      featureName\n      createdAt\n      updatedAt\n      readAt\n    }\n  }\n',
 'updateclicllink': '\n  mutation UpdateCliclLink($input: UpdateCLICLLinkInput!) {\n    updateCLICLLink(input: $input) {\n      id\n      createdAt\n      updatedAt\n      CheckListItemID\n      lane\n      CheckListID\n      CheckList {\n        id\n        createdAt\n        updatedAt\n        name\n        description\n        type\n        content\n        author\n      }\n      CheckListItem {\n        id\n        localID\n        createdAt\n        updatedAt\n        author\n        content\n        title\n        type\n        connection\n        MMNodeConnection\n        investigationID\n        assignee\n        associatedPlaybooks\n        associatedCommands\n      }\n    }\n  }\n',
 'updatecommands': '\n  mutation UpdateCommands($input: UpdateCommandsInput!) {\n    updateCommands(input: $input) {\n      id\n      from\n      mode\n      command\n      output\n      state\n      returnCode\n      investigationId\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'updatecomment': '\n  mutation UpdateComment($input: UpdateCommentInput!) {\n    updateComment(input: $input) {\n      id\n      parent\n      createdAt\n      updatedAt\n      author\n      content\n      type\n      links\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'updateconferenceconfig': '\n  mutation UpdateConferenceConfig($input: UpdateConferenceConfigInput!) {\n    updateConferenceConfig(input: $input) {\n      id\n      url\n      userName\n      password\n      author\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updateconnectorform': '\n  mutation UpdateConnectorForm($input: UpdateConnectorFormInput!) {\n    updateConnectorForm(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      owner\n      connectorID\n      connector {\n        id\n        createdAt\n        updatedAt\n        category\n        name\n        owner\n        description\n        detaileddescription\n        iconPath\n        version\n        md5\n        source\n        configuration\n        commands\n        commandsType\n        script\n        scriptType\n        scriptPath\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n      configData\n      commandData\n      nodeData\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n      runbooks {\n        nextToken\n      }\n    }\n  }\n',
 'updateconnectors': '\n  mutation UpdateConnectors($input: UpdateConnectorsInput!) {\n    updateConnectors(input: $input) {\n      id\n      createdAt\n      updatedAt\n      category\n      name\n      owner\n      description\n      detaileddescription\n      iconPath\n      version\n      md5\n      source\n      configuration\n      commands\n      commandsType\n      script\n      scriptType\n      scriptPath\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n    }\n  }\n',
 'updatedashboardconfig': '\n  mutation UpdateDashboardConfig($input: UpdateDashboardConfigInput!) {\n    updateDashboardConfig(input: $input) {\n      id\n      url\n      labels {\n        nextToken\n      }\n      author\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updatedashboardconfiglabellink': '\n  mutation UpdateDashboardConfigLabelLink(\n    $input: UpdateDashboardConfigLabelLinkInput!\n  ) {\n    updateDashboardConfigLabelLink(input: $input) {\n      id\n      dashboardConfigID\n      labelID\n      dashboardConfig {\n        id\n        url\n        author\n        createdAt\n        updatedAt\n      }\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updateecubesandboxexecution': '\n  mutation UpdateEcubeSandboxExecution(\n    $input: UpdateEcubeSandboxExecutionInput!\n  ) {\n    updateEcubeSandboxExecution(input: $input) {\n      id\n      owner\n      output\n      outputType\n      returnCode\n      status\n      createdAt\n      updatedAt\n      E3One\n      E3Two\n      E3Three\n    }\n  }\n',
 'updateepicautocomplete': '\n  mutation UpdateEpicAutoComplete($input: UpdateEpicAutoCompleteInput!) {\n    updateEpicAutoComplete(input: $input) {\n      id\n      author\n      cliCommand\n      command\n      createdAt\n      updatedAt\n      investigationId\n      times\n    }\n  }\n',
 'updateepiclicommand': '\n  mutation UpdateEpicliCommand($input: UpdateEPICLICommandInput!) {\n    updateEPICLICommand(input: $input) {\n      id\n      createdAt\n      updatedAt\n      author\n      content\n      type\n    }\n  }\n',
 'updateexecutedrunbooks': '\n  mutation UpdateExecutedRunbooks($input: UpdateExecutedRunbooksInput!) {\n    updateExecutedRunbooks(input: $input) {\n      id\n      createdAt\n      updatedAt\n      runbookID\n      investigationId\n      output\n      outputType\n      title\n      state\n      author\n      rbVars\n      callbackUrl\n      callbackData\n      source\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n      runbook {\n        id\n        name\n        description\n        content\n        author\n        commands\n        createdAt\n        updatedAt\n        RunBookConnectors\n        type\n        longSave\n        rbVars\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n    }\n  }\n',
 'updateexecutedworkflowstate': '\n  mutation UpdateExecutedWorkflowState(\n    $input: UpdateExecutedWorkflowStateInput!\n  ) {\n    updateExecutedWorkflowState(input: $input) {\n      id\n      createdAt\n      updatedAt\n      name\n      state\n      output\n      type\n      allworkflowID\n      investigationID\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'updategroupmember': '\n  mutation UpdateGroupMember($input: UpdateGroupMemberInput!) {\n    updateGroupMember(input: $input) {\n      id\n      hostId\n      groupId\n      host {\n        id\n        inventoryId\n        createdAt\n        updatedAt\n        author\n        host\n        port\n        vars\n      }\n      group {\n        id\n        inventoryId\n        createdAt\n        updatedAt\n        author\n        name\n        groups\n        port\n        vars\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updateguardrail': '\n  mutation UpdateGuardRail($input: UpdateGuardRailInput!) {\n    updateGuardRail(input: $input) {\n      id\n      timeOut\n      message\n      labels {\n        nextToken\n      }\n      author\n      active\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updateguardraillabellink': '\n  mutation UpdateGuardRailLabelLink($input: UpdateGuardRailLabelLinkInput!) {\n    updateGuardRailLabelLink(input: $input) {\n      id\n      guardRailID\n      labelID\n      guardRail {\n        id\n        timeOut\n        message\n        author\n        active\n        createdAt\n        updatedAt\n      }\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updatehint': '\n  mutation UpdateHint($input: UpdateHintInput!) {\n    updateHint(input: $input) {\n      id\n      investigationId\n      createdAt\n      updatedAt\n      type\n      connection\n      pastInvestigationID\n      title\n      importance\n      starrted\n      applied\n      url\n      why\n    }\n  }\n',
 'updateinventory': '\n  mutation UpdateInventory($input: UpdateInventoryInput!) {\n    updateInventory(input: $input) {\n      id\n      createdAt\n      updatedAt\n      author\n      name\n      all {\n        nextToken\n      }\n      groups {\n        nextToken\n      }\n      vars\n    }\n  }\n',
 'updateinventorygroup': '\n  mutation UpdateInventoryGroup($input: UpdateInventoryGroupInput!) {\n    updateInventoryGroup(input: $input) {\n      id\n      inventoryId\n      createdAt\n      updatedAt\n      author\n      name\n      hosts {\n        nextToken\n      }\n      groups\n      port\n      vars\n      inventory {\n        id\n        createdAt\n        updatedAt\n        author\n        name\n        vars\n      }\n    }\n  }\n',
 'updateinventoryhost': '\n  mutation UpdateInventoryHost($input: UpdateInventoryHostInput!) {\n    updateInventoryHost(input: $input) {\n      id\n      inventoryId\n      createdAt\n      updatedAt\n      author\n      host\n      port\n      vars\n      groups {\n        nextToken\n      }\n      inventory {\n        id\n        createdAt\n        updatedAt\n        author\n        name\n        vars\n      }\n    }\n  }\n',
 'updateinvestigation': '\n  mutation UpdateInvestigation($input: UpdateInvestigationInput!) {\n    updateInvestigation(input: $input) {\n      id\n      title\n      from\n      createdAt\n      currentState\n      HumanID\n      currentStateID\n      severity\n      priority\n      location\n      labels {\n        nextToken\n      }\n      channelId\n      investigationChannel {\n        id\n        name\n        title\n        members\n        membersState\n        createdAt\n        updatedAt\n        memberLocation\n      }\n      updatedAt\n      resolvedTime\n      resolvedComment\n      Symptoms\n      MMNodeLinks {\n        nextToken\n      }\n      CheckListInvLink {\n        nextToken\n      }\n      meetingType\n      meetingId\n    }\n  }\n',
 'updateinvestigationevent': '\n  mutation UpdateInvestigationEvent($input: UpdateInvestigationEventInput!) {\n    updateInvestigationEvent(input: $input) {\n      id\n      createdAt\n      updatedAt\n      author\n      content\n      type\n      links\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'updateinvestigationguestuserlink': '\n  mutation UpdateInvestigationGuestUserLink(\n    $input: UpdateInvestigationGuestUserLinkInput!\n  ) {\n    updateInvestigationGuestUserLink(input: $input) {\n      id\n      userName\n      createdAt\n      updatedAt\n      investigationID\n      channelID\n    }\n  }\n',
 'updatelabel': '\n  mutation UpdateLabel($input: UpdateLabelInput!) {\n    updateLabel(input: $input) {\n      id\n      name\n      color\n      type\n      investigations {\n        nextToken\n      }\n      userSettings {\n        nextToken\n      }\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updatelabellink': '\n  mutation UpdateLabelLink($input: UpdateLabelLinkInput!) {\n    updateLabelLink(input: $input) {\n      id\n      investigationID\n      labelID\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'updateleadboard': '\n  mutation UpdateLeadBoard($input: UpdateLeadBoardInput!) {\n    updateLeadBoard(input: $input) {\n      id\n      investigationId\n      content\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'updateleadboardnode': '\n  mutation UpdateLeadBoardNode($input: UpdateLeadBoardNodeInput!) {\n    updateLeadBoardNode(input: $input) {\n      id\n      localID\n      createdAt\n      updatedAt\n      author\n      content\n      title\n      notes\n      associatedPlaybooks\n      associatedCommands\n      InvestLinks {\n        nextToken\n      }\n    }\n  }\n',
 'updatemessage': '\n  mutation UpdateMessage($input: UpdateMessageInput!) {\n    updateMessage(input: $input) {\n      id\n      authorId\n      content\n      type\n      channel {\n        id\n        name\n        title\n        members\n        membersState\n        createdAt\n        updatedAt\n        memberLocation\n      }\n      messageChannelId\n      createdAt\n      updatedAt\n      author {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n    }\n  }\n',
 'updatemmnodelink': '\n  mutation UpdateMmNodeLink($input: UpdateMMNodeLinkInput!) {\n    updateMMNodeLink(input: $input) {\n      id\n      createdAt\n      updatedAt\n      assigned\n      state\n      content\n      author\n      investigationID\n      connectionToTask\n      LBNID\n      LeadBoardNodeId {\n        id\n        localID\n        createdAt\n        updatedAt\n        author\n        content\n        title\n        notes\n        associatedPlaybooks\n        associatedCommands\n      }\n      investigationId {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'updatenotification': '\n  mutation UpdateNotification($input: UpdateNotificationInput!) {\n    updateNotification(input: $input) {\n      id\n      userName\n      content\n      containerType\n      containerId\n      investigationId\n      isViewed\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updatepin': '\n  mutation UpdatePin($input: UpdatePinInput!) {\n    updatePin(input: $input) {\n      id\n      imageId\n      content\n      createdAt\n      updatedAt\n      author\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'updateprivatekey': '\n  mutation UpdatePrivateKey($input: UpdatePrivateKeyInput!) {\n    updatePrivateKey(input: $input) {\n      id\n      sshKey\n      userName\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updateremotecontrol': '\n  mutation UpdateRemoteControl($input: UpdateRemoteControlInput!) {\n    updateRemoteControl(input: $input) {\n      id\n      from\n      to\n      command\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updatereportsummaryeditor': '\n  mutation UpdateReportSummaryEditor($input: UpdateReportSummaryEditorInput!) {\n    updateReportSummaryEditor(input: $input) {\n      id\n      investigationId\n      createdAt\n      updatedAt\n      author\n      content\n      type\n    }\n  }\n',
 'updaterunbookconnectorformlink': '\n  mutation UpdateRunbookConnectorFormLink(\n    $input: UpdateRunbookConnectorFormLinkInput!\n  ) {\n    updateRunbookConnectorFormLink(input: $input) {\n      id\n      connectorFormId\n      runbookId\n      connectorForm {\n        id\n        createdAt\n        updatedAt\n        name\n        owner\n        connectorID\n        configData\n        commandData\n        nodeData\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n      createdAt\n      updatedAt\n      runbook {\n        id\n        name\n        description\n        content\n        author\n        commands\n        createdAt\n        updatedAt\n        RunBookConnectors\n        type\n        longSave\n        rbVars\n        scope\n        PeopleCanView\n        PeopleCanAccess\n        GroupsCanView\n        GroupsCanAccess\n      }\n    }\n  }\n',
 'updateschedpbs': '\n  mutation UpdateSchedPBs($input: UpdateSchedPBsInput!) {\n    updateSchedPBs(input: $input) {\n      id\n      pbId\n      frequency\n      pbName\n      rbVars\n      scheduled\n      createdAt\n      updatedAt\n      author\n    }\n  }\n',
 'updateslackchannelinfo': '\n  mutation UpdateSlackChannelInfo($input: UpdateSlackChannelInfoInput!) {\n    updateSlackChannelInfo(input: $input) {\n      id\n      owner\n      slackChannelId\n      slackAppId\n      slackTeamId\n      createdAt\n      updatedAt\n      slackTeam {\n        id\n        owner\n        teamId\n        teamName\n        slackAppId\n        botId\n        botToken\n        botScope\n        enterprise\n        createdAt\n        updatedAt\n      }\n      usersEnabled {\n        nextToken\n      }\n    }\n  }\n',
 'updateslackenabledchannelsuserlink': '\n  mutation UpdateSlackEnabledChannelsUserLink(\n    $input: UpdateSlackEnabledChannelsUserLinkInput!\n  ) {\n    updateSlackEnabledChannelsUserLink(input: $input) {\n      id\n      owner\n      slackUserId\n      slackChannelId\n      createdAt\n      updatedAt\n      slackUser {\n        id\n        userAppSyncId\n        userName\n        userId\n        userScope\n        userToken\n        slackAppId\n        slackTeamId\n        createdAt\n        updatedAt\n      }\n      slackChannel {\n        id\n        owner\n        slackChannelId\n        slackAppId\n        slackTeamId\n        createdAt\n        updatedAt\n      }\n    }\n  }\n',
 'updateslackteaminfo': '\n  mutation UpdateSlackTeamInfo($input: UpdateSlackTeamInfoInput!) {\n    updateSlackTeamInfo(input: $input) {\n      id\n      owner\n      teamId\n      teamName\n      slackAppId\n      botId\n      botToken\n      botScope\n      enterprise\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updateslackuserinfo': '\n  mutation UpdateSlackUserInfo($input: UpdateSlackUserInfoInput!) {\n    updateSlackUserInfo(input: $input) {\n      id\n      userAppSyncId\n      userName\n      userId\n      userScope\n      userToken\n      slackAppId\n      slackTeamId\n      createdAt\n      updatedAt\n      userGqlInfo {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n      slackTeam {\n        id\n        owner\n        teamId\n        teamName\n        slackAppId\n        botId\n        botToken\n        botScope\n        enterprise\n        createdAt\n        updatedAt\n      }\n      channelsEnabled {\n        nextToken\n      }\n    }\n  }\n',
 'updatesshtabs': '\n  mutation UpdateSshTabs($input: UpdateSshTabsInput!) {\n    updateSshTabs(input: $input) {\n      id\n      investigationId\n      host\n      author\n      userName\n      createdAt\n      updatedAt\n    }\n  }\n',
 'updatestash': '\n  mutation UpdateStash($input: UpdateStashInput!) {\n    updateStash(input: $input) {\n      id\n      Key\n      Value\n      createdAt\n      updatedAt\n      author\n      scope\n      PeopleCanView\n      PeopleCanAccess\n      GroupsCanView\n      GroupsCanAccess\n    }\n  }\n',
 'updatestate': '\n  mutation UpdateState($input: UpdateStateInput!) {\n    updateState(input: $input) {\n      id\n      name\n      createdAt\n      updatedAt\n      author\n      lastState\n      investigationId\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'updatesummaryinvestigation': '\n  mutation UpdateSummaryInvestigation(\n    $input: UpdateSummaryInvestigationInput!\n  ) {\n    updateSummaryInvestigation(input: $input) {\n      id\n      createdAt\n      updatedAt\n      resolvedTime\n      OriginalID\n      title\n      HumanID\n      Commands\n      InvestigationEvents\n      CompleteInvestigationEvents\n      Participants\n      Comments\n      Pins\n      AvailableRunbooks\n      UsedRunbooks\n      Hints\n      Symptoms\n      owner\n    }\n  }\n',
 'updatetenantcustomer': '\n  mutation UpdateTenantCustomer($input: UpdateTenantCustomerInput!) {\n    updateTenantCustomer(input: $input) {\n      label\n      impactLabels\n      createdAt\n      updatedAt\n      customerName\n      email\n      phoneno\n      SLAType\n      city\n      state\n      contry\n      zipcode\n    }\n  }\n',
 'updatetranscriptionrecords': '\n  mutation UpdateTranscriptionRecords(\n    $input: UpdateTranscriptionRecordsInput!\n  ) {\n    updateTranscriptionRecords(input: $input) {\n      id\n      investigationId\n      createdAt\n      updatedAt\n      spokenBy\n      content\n    }\n  }\n',
 'updateusedtogenhintalready': '\n  mutation UpdateUsedToGenHintAlready(\n    $input: UpdateUsedToGenHintAlreadyInput!\n  ) {\n    updateUsedToGenHintAlready(input: $input) {\n      id\n      createdAt\n      updatedAt\n      investigationId\n      commands\n      text\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n    }\n  }\n',
 'updateuser': '\n  mutation UpdateUser($input: UpdateUserInput!) {\n    updateUser(input: $input) {\n      id\n      username\n      email\n      firstname\n      lastname\n      avatar\n      mood\n      name\n      role\n      status\n      timezone\n      phone\n      address\n      location\n      chatList {\n        nextToken\n      }\n      messages {\n        nextToken\n      }\n      createdAt\n      updatedAt\n      owner\n    }\n  }\n',
 'updateuseractivity': '\n  mutation UpdateUserActivity($input: UpdateUserActivityInput!) {\n    updateUserActivity(input: $input) {\n      id\n      userID\n      status\n      investigationID\n      createdAt\n      updatedAt\n      investigation {\n        id\n        title\n        from\n        createdAt\n        currentState\n        HumanID\n        currentStateID\n        severity\n        priority\n        location\n        channelId\n        updatedAt\n        resolvedTime\n        resolvedComment\n        Symptoms\n        meetingType\n        meetingId\n      }\n      user {\n        id\n        username\n        email\n        firstname\n        lastname\n        avatar\n        mood\n        name\n        role\n        status\n        timezone\n        phone\n        address\n        location\n        createdAt\n        updatedAt\n        owner\n      }\n    }\n  }\n',
 'updateusersetting': '\n  mutation UpdateUserSetting($input: UpdateUserSettingInput!) {\n    updateUserSetting(input: $input) {\n      id\n      userId\n      owner\n      createdAt\n      updatedAt\n      labels {\n        nextToken\n      }\n      value\n      url\n      name\n      type\n      showWelcomeNext\n      showWelcomeSkip\n      showWelcomeRemind\n      welcomeData\n    }\n  }\n',
 'updateusersettinglabellink': '\n  mutation UpdateUserSettingLabelLink(\n    $input: UpdateUserSettingLabelLinkInput!\n  ) {\n    updateUserSettingLabelLink(input: $input) {\n      id\n      userSettingId\n      userSetting {\n        id\n        userId\n        owner\n        createdAt\n        updatedAt\n        value\n        url\n        name\n        type\n        showWelcomeNext\n        showWelcomeSkip\n        showWelcomeRemind\n        welcomeData\n      }\n      labelID\n      label {\n        id\n        name\n        color\n        type\n        createdAt\n        updatedAt\n      }\n      type\n      createdAt\n      updatedAt\n    }\n  }\n'}
